@@ -62,7 +62,10 @@ fun Routing.coreBankApi(db: Database, ctx: BankConfig) {
                 supported_tan_channels = ctx.tanChannels.keys,
                 allow_edit_name = ctx.allowEditName,
                 allow_edit_cashout_payto_uri = ctx.allowEditCashout,
-                wire_type = ctx.wireMethod
+                wire_type = ctx.wireMethod,
+                wire_transfer_fees = ctx.wireTransferFees,
+                min_wire_transfer_amount = ctx.minAmount,
+                max_wire_transfer_amount = ctx.maxAmount
             )
         )
     }
@@ -89,12 +92,12 @@ private fun Routing.coreBankTokenApi(db: Database) {
             
             if (existingToken != null) {
                 // This block checks permissions ONLY IF the call was authenticated with a token
-                val refreshingToken = db.token.get(existingToken) ?: throw internalServerError(
+                val refreshingToken = db.token.access(existingToken, Instant.now()) ?: throw internalServerError(
                     "Token used to auth not found in the database!"
                 )
-                if (refreshingToken.scope == TokenScope.readonly && req.scope == TokenScope.readwrite)
+                if (!validScope(req.scope, refreshingToken.scope))
                     throw forbidden(
-                        "Cannot generate RW token from RO",
+                        "Impossible to refresh a token with a larger scope",
                         TalerErrorCode.GENERIC_TOKEN_PERMISSION_INSUFFICIENT
                     )
             }
@@ -120,7 +123,8 @@ private fun Routing.coreBankTokenApi(db: Database) {
                 creationTime = creationTime,
                 expirationTime = expirationTimestamp,
                 scope = req.scope,
-                isRefreshable = req.refreshable
+                isRefreshable = req.refreshable,
+                description = req.description
             )) {
                 throw internalServerError("Failed at inserting new token in the database")
             }
@@ -138,6 +142,15 @@ private fun Routing.coreBankTokenApi(db: Database) {
             db.token.delete(token)
             call.respond(HttpStatusCode.NoContent)
         }
+        get("/accounts/{USERNAME}/tokens") {
+            val params = PageParams.extract(call.request.queryParameters)
+            val tokens = db.token.page(params, username)
+            if (tokens.isEmpty()) {
+                call.respond(HttpStatusCode.NoContent)
+            } else {
+                call.respond(TokenInfos(tokens))
+            }
+        }
     }
 }
 
@@ -174,7 +187,7 @@ suspend fun createAccount(
             )
 
     } else if (req.tan_channel != null) {
-        if (cfg.tanChannels.get(req.tan_channel) == null) {
+        if (cfg.tanChannels[req.tan_channel] == null) {
             throw unsupportedTanChannel(req.tan_channel)
         } 
         val missing = when (req.tan_channel) {
@@ -246,7 +259,7 @@ suspend fun patchAccount(
     db: Database, 
     cfg: BankConfig, 
     req: AccountReconfiguration, 
-    username: String, 
+    login: String, 
     isAdmin: Boolean, 
     is2fa: Boolean, 
     channel: TanChannel? = null, 
@@ -254,7 +267,7 @@ suspend fun patchAccount(
 ): AccountPatchResult {
     req.debit_threshold?.run { cfg.checkRegionalCurrency(this) }
 
-    if (username == "admin" && req.is_public == true)
+    if (login == "admin" && req.is_public == true)
         throw conflict(
             "'admin' account cannot be public",
             TalerErrorCode.END
@@ -265,7 +278,7 @@ suspend fun patchAccount(
     }
 
     return db.account.reconfig( 
-        login = username,
+        login = login,
         name = req.name,
         cashoutPayto = req.cashout_payto_uri, 
         email = req.contact_data?.email ?: Option.None,
@@ -455,7 +468,10 @@ private fun Routing.coreBankTransactionsApi(db: Database, ctx: BankConfig) {
                 amount = amount,
                 timestamp = Instant.now(),
                 requestUid = req.request_uid,
-                is2fa = challenge != null
+                is2fa = challenge != null,
+                wireTransferFees = ctx.wireTransferFees,
+                minAmount = ctx.minAmount,
+                maxAmount = ctx.maxAmount
             )
             when (res) {
                 BankTransactionResult.UnknownDebtor -> throw unknownAccount(username)
@@ -475,6 +491,10 @@ private fun Routing.coreBankTransactionsApi(db: Database, ctx: BankConfig) {
                     "Insufficient funds",
                     TalerErrorCode.BANK_UNALLOWED_DEBIT
                 )
+                BankTransactionResult.BadAmount -> throw conflict(
+                    "Amount either to high or too low",
+                    TalerErrorCode.BANK_UNALLOWED_DEBIT
+                )
                 BankTransactionResult.RequestUidReuse -> throw conflict(
                     "request_uid used already",
                     TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED
@@ -489,9 +509,19 @@ private fun Routing.coreBankWithdrawalApi(db: Database, ctx: BankConfig) {
     auth(db, TokenScope.readwrite) {
         post("/accounts/{USERNAME}/withdrawals") {
             val req = call.receive<BankAccountCreateWithdrawalRequest>()
-            ctx.checkRegionalCurrency(req.amount)
+            req.amount?.run(ctx::checkRegionalCurrency)
+            req.suggested_amount?.run(ctx::checkRegionalCurrency)
             val opId = UUID.randomUUID()
-            when (db.withdrawal.create(username, opId, req.amount, Instant.now())) {
+            when (db.withdrawal.create(
+                username, 
+                opId, 
+                req.amount,
+                req.suggested_amount,
+                Instant.now(),
+                ctx.wireTransferFees,
+                ctx.minAmount,
+                ctx.maxAmount
+            )) {
                 WithdrawalCreationResult.UnknownAccount -> throw unknownAccount(username)
                 WithdrawalCreationResult.AccountIsExchange -> throw conflict(
                     "Exchange account cannot perform withdrawal operation",
@@ -501,6 +531,10 @@ private fun Routing.coreBankWithdrawalApi(db: Database, ctx: BankConfig) {
                     "Insufficient funds to withdraw with Taler",
                     TalerErrorCode.BANK_UNALLOWED_DEBIT
                 )
+                WithdrawalCreationResult.BadAmount -> throw conflict(
+                    "Amount either to high or too low",
+                    TalerErrorCode.BANK_UNALLOWED_DEBIT
+                )
                 WithdrawalCreationResult.Success -> {
                     call.respond(
                         BankAccountCreateWithdrawalResponse(
@@ -514,7 +548,15 @@ private fun Routing.coreBankWithdrawalApi(db: Database, ctx: BankConfig) {
         post("/accounts/{USERNAME}/withdrawals/{withdrawal_id}/confirm") {
             val id = call.uuidPath("withdrawal_id")
             val challenge = call.checkChallenge(db, Operation.withdrawal)
-            when (db.withdrawal.confirm(username, id, Instant.now(), challenge != null)) {
+            when (db.withdrawal.confirm(
+                username,
+                id,
+                Instant.now(),
+                challenge != null,
+                ctx.wireTransferFees,
+                ctx.minAmount,
+                ctx.maxAmount
+            )) {
                 WithdrawalConfirmationResult.UnknownOperation -> throw notFound(
                     "Withdrawal operation $id not found",
                     TalerErrorCode.BANK_TRANSACTION_NOT_FOUND
@@ -531,6 +573,10 @@ private fun Routing.coreBankWithdrawalApi(db: Database, ctx: BankConfig) {
                     "Insufficient funds",
                     TalerErrorCode.BANK_UNALLOWED_DEBIT
                 )
+                WithdrawalConfirmationResult.BadAmount -> throw conflict(
+                    "Amount either to high or too low",
+                    TalerErrorCode.BANK_UNALLOWED_DEBIT
+                )
                 WithdrawalConfirmationResult.UnknownExchange -> throw conflict(
                     "Exchange to withdraw from not found",
                     TalerErrorCode.BANK_UNKNOWN_CREDITOR
@@ -581,7 +627,7 @@ private fun Routing.coreBankCashoutApi(db: Database, ctx: BankConfig) = conditio
                 amountDebit = req.amount_debit, 
                 amountCredit = req.amount_credit, 
                 subject = req.subject ?: "", // TODO default subject
-                now = Instant.now(),
+                timestamp = Instant.now(),
                 is2fa = challenge != null
             )
             when (res) {
@@ -657,7 +703,7 @@ private fun Routing.coreBankTanApi(db: Database, ctx: BankConfig) {
                 id = id,
                 login = username,
                 code = Tan.genCode(),
-                now = Instant.now(), 
+                timestamp = Instant.now(), 
                 retryCounter = TAN_RETRY_COUNTER,
                 validityPeriod = TAN_VALIDITY_PERIOD
             )
@@ -668,7 +714,7 @@ private fun Routing.coreBankTanApi(db: Database, ctx: BankConfig) {
                 )
                 is TanSendResult.Success -> {
                     res.tanCode?.run {
-                        val (tanScript, tanEnv) = ctx.tanChannels.get(res.tanChannel) 
+                        val (tanScript, tanEnv) = ctx.tanChannels[res.tanChannel]
                             ?: throw unsupportedTanChannel(res.tanChannel)
                         val msg = "T-${res.tanCode} is your ${ctx.name} verification code"
                         val exitValue = withContext(Dispatchers.IO) {
@@ -687,7 +733,7 @@ private fun Routing.coreBankTanApi(db: Database, ctx: BankConfig) {
                             val exitValue = process.exitValue()
                             if (exitValue != 0) {
                                 val out = runCatching {
-                                    process.getInputStream().use {
+                                    process.inputStream.use {
                                         reader().readText()
                                     }
                                 }.getOrDefault("")
@@ -721,7 +767,7 @@ private fun Routing.coreBankTanApi(db: Database, ctx: BankConfig) {
                 id = id,
                 login = username,
                 code = code,
-                now = Instant.now()
+                timestamp = Instant.now()
             )
             when (res) {
                 TanSolveResult.NotFound -> throw notFound(
