@@ -52,7 +52,7 @@ fun Routing.wireGatewayApi(db: Database, ctx: BankConfig) {
             val res = db.exchange.transfer(
                 req = req,
                 login = username,
-                now = Instant.now()
+                timestamp = Instant.now()
             )
             when (res) {
                 is TransferResult.UnknownExchange -> throw unknownAccount(username)
@@ -119,7 +119,7 @@ fun Routing.wireGatewayApi(db: Database, ctx: BankConfig) {
             val res = db.exchange.addIncoming(
                 req = req,
                 login = username,
-                now = timestamp
+                timestamp = timestamp
             )
             when (res) {
                 is AddIncomingResult.UnknownExchange -> throw unknownAccount(username)
