@@ -1,6 +1,6 @@
 /*
  * This file is part of LibEuFin.
- * Copyright (C) 2023 Taler Systems S.A.
+ * Copyright (C) 2023-2024 Taler Systems S.A.
 
  * LibEuFin is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -47,137 +47,140 @@ class AccountDAO(private val db: Database) {
         isPublic: Boolean,
         isTalerExchange: Boolean,
         maxDebt: TalerAmount,
+        minCashout: TalerAmount?,
         bonus: TalerAmount,
         tanChannel: TanChannel?,
         // Whether to check [internalPaytoUri] for idempotency
         checkPaytoIdempotent: Boolean,
         ctx: BankPaytoCtx
-    ): AccountCreationResult = db.serializable { it ->
-        val now = Instant.now().micros()
-        it.transaction { conn ->
-            val idempotent = conn.prepareStatement("""
-                SELECT password_hash, name=?
-                    AND email IS NOT DISTINCT FROM ?
-                    AND phone IS NOT DISTINCT FROM ?
-                    AND cashout_payto IS NOT DISTINCT FROM ?
-                    AND (NOT ? OR internal_payto_uri=?)
-                    AND is_public=?
-                    AND is_taler_exchange=?
-                    AND tan_channel IS NOT DISTINCT FROM ?::tan_enum
-                    ,internal_payto_uri, name
-                FROM customers 
-                    JOIN bank_accounts
-                        ON customer_id=owning_customer_id
-                WHERE login=?
-            """).run {
-                // TODO check max debt
-                setString(1, name)
-                setString(2, email)
-                setString(3, phone)
-                setString(4, cashoutPayto?.full(name))
-                setBoolean(5, checkPaytoIdempotent)
-                setString(6, internalPayto.canonical)
-                setBoolean(7, isPublic)
-                setBoolean(8, isTalerExchange)
-                setString(9, tanChannel?.name)
-                setString(10, login)
-                oneOrNull { 
-                    Pair(
-                        PwCrypto.checkpw(password, it.getString(1)) && it.getBoolean(2),
-                        it.getBankPayto("internal_payto_uri", "name", ctx)
-                    )
-                } 
-            }
-            
-            if (idempotent != null) {
-                if (idempotent.first) {
-                    AccountCreationResult.Success(idempotent.second)
-                } else {
-                    AccountCreationResult.LoginReuse
-                }
+    ): AccountCreationResult = db.serializableTransaction { conn ->
+        val timestamp = Instant.now().micros()
+        val idempotent = conn.withStatement("""
+            SELECT password_hash, name=?
+                AND email IS NOT DISTINCT FROM ?
+                AND phone IS NOT DISTINCT FROM ?
+                AND cashout_payto IS NOT DISTINCT FROM ?
+                AND (NOT ? OR internal_payto_uri=?)
+                AND is_public=?
+                AND is_taler_exchange=?
+                AND tan_channel IS NOT DISTINCT FROM ?::tan_enum
+                ,internal_payto_uri, name
+            FROM customers 
+                JOIN bank_accounts
+                    ON customer_id=owning_customer_id
+            WHERE login=?
+        """) {
+            // TODO check max debt and min checkout ?
+            setString(1, name)
+            setString(2, email)
+            setString(3, phone)
+            setString(4, cashoutPayto?.full(name))
+            setBoolean(5, checkPaytoIdempotent)
+            setString(6, internalPayto.canonical)
+            setBoolean(7, isPublic)
+            setBoolean(8, isTalerExchange)
+            setString(9, tanChannel?.name)
+            setString(10, login)
+            oneOrNull { 
+                Pair(
+                    PwCrypto.checkpw(password, it.getString(1)).match && it.getBoolean(2),
+                    it.getBankPayto("internal_payto_uri", "name", ctx)
+                )
+            } 
+        }
+        
+        if (idempotent != null) {
+            if (idempotent.first) {
+                AccountCreationResult.Success(idempotent.second)
             } else {
-                if (internalPayto is IbanPayto)
-                    conn.prepareStatement("""
-                        INSERT INTO iban_history(
-                            iban
-                            ,creation_time
-                        ) VALUES (?, ?)
-                    """).run {
-                        setString(1, internalPayto.iban.value)
-                        setLong(2, now)
-                        if (!executeUpdateViolation()) {
-                            conn.rollback()
-                            return@transaction AccountCreationResult.PayToReuse
-                        }
-                    }
-
-                val customerId = conn.prepareStatement("""
-                    INSERT INTO customers (
-                        login
-                        ,password_hash
-                        ,name
-                        ,email
-                        ,phone
-                        ,cashout_payto
-                        ,tan_channel
-                    ) VALUES (?, ?, ?, ?, ?, ?, ?::tan_enum)
-                        RETURNING customer_id
-                """
-                ).run {
-                    setString(1, login)
-                    setString(2, PwCrypto.hashpw(password))
-                    setString(3, name)
-                    setString(4, email)
-                    setString(5, phone)
-                    setString(6, cashoutPayto?.full(name))
-                    setString(7, tanChannel?.name)
-                    oneOrNull { it.getLong("customer_id") }!!
-                }
-            
-                conn.prepareStatement("""
-                    INSERT INTO bank_accounts(
-                        internal_payto_uri
-                        ,owning_customer_id
-                        ,is_public
-                        ,is_taler_exchange
-                        ,max_debt
-                    ) VALUES (?, ?, ?, ?, (?, ?)::taler_amount)
-                """).run {
-                    setString(1, internalPayto.canonical)
-                    setLong(2, customerId)
-                    setBoolean(3, isPublic)
-                    setBoolean(4, isTalerExchange)
-                    setLong(5, maxDebt.value)
-                    setInt(6, maxDebt.frac)
+                AccountCreationResult.LoginReuse
+            }
+        } else {
+            if (internalPayto is IbanPayto)
+                conn.withStatement ("""
+                    INSERT INTO iban_history(
+                        iban
+                        ,creation_time
+                    ) VALUES (?, ?)
+                """) {
+                    setString(1, internalPayto.iban.value)
+                    setLong(2, timestamp)
                     if (!executeUpdateViolation()) {
                         conn.rollback()
-                        return@transaction AccountCreationResult.PayToReuse
+                        return@serializableTransaction AccountCreationResult.PayToReuse
                     }
                 }
-                
-                if (bonus.value != 0L || bonus.frac != 0) {
-                    conn.prepareStatement("""
-                        SELECT out_balance_insufficient
-                        FROM bank_transaction(?,'admin','bonus',(?,?)::taler_amount,?,true,NULL)
-                    """).run {
-                        setString(1, internalPayto.canonical)
-                        setLong(2, bonus.value)
-                        setInt(3, bonus.frac)
-                        setLong(4, now)
-                        executeQuery().use {
-                            when {
-                                !it.next() -> throw internalServerError("Bank transaction didn't properly return")
-                                it.getBoolean("out_balance_insufficient") -> {
-                                    conn.rollback()
-                                    AccountCreationResult.BonusBalanceInsufficient
-                                }
-                                else -> AccountCreationResult.Success(internalPayto.bank(name, ctx))
+
+            val customerId = conn.withStatement("""
+                INSERT INTO customers (
+                    login
+                    ,password_hash
+                    ,name
+                    ,email
+                    ,phone
+                    ,cashout_payto
+                    ,tan_channel
+                ) VALUES (?, ?, ?, ?, ?, ?, ?::tan_enum)
+                    RETURNING customer_id
+            """
+            ) {
+                setString(1, login)
+                setString(2, PwCrypto.hashpw(password))
+                setString(3, name)
+                setString(4, email)
+                setString(5, phone)
+                setString(6, cashoutPayto?.full(name))
+                setString(7, tanChannel?.name)
+                one { it.getLong("customer_id") }
+            }
+        
+            conn.withStatement("""
+                INSERT INTO bank_accounts(
+                    internal_payto_uri
+                    ,owning_customer_id
+                    ,is_public
+                    ,is_taler_exchange
+                    ,max_debt
+                    ,min_cashout
+                ) VALUES (?, ?, ?, ?, (?, ?)::taler_amount, ${if (minCashout == null) "NULL" else "(?, ?)::taler_amount"})
+            """) {
+                setString(1, internalPayto.canonical)
+                setLong(2, customerId)
+                setBoolean(3, isPublic)
+                setBoolean(4, isTalerExchange)
+                setLong(5, maxDebt.value)
+                setInt(6, maxDebt.frac)
+                if (minCashout != null) {
+                    setLong(7, minCashout.value)
+                    setInt(8, minCashout.frac)
+                }
+                if (!executeUpdateViolation()) {
+                    conn.rollback()
+                    return@serializableTransaction AccountCreationResult.PayToReuse
+                }
+            }
+            
+            if (bonus.value != 0L || bonus.frac != 0) {
+                conn.withStatement("""
+                    SELECT out_balance_insufficient
+                    FROM bank_transaction(?,'admin','bonus',(?,?)::taler_amount,?,true,NULL,NULL,NULL,NULL)
+                """) {
+                    setString(1, internalPayto.canonical)
+                    setLong(2, bonus.value)
+                    setInt(3, bonus.frac)
+                    setLong(4, timestamp)
+                    one {
+                        when {
+                            it.getBoolean("out_balance_insufficient") -> {
+                                conn.rollback()
+                                AccountCreationResult.BonusBalanceInsufficient
                             }
+                            else -> AccountCreationResult.Success(internalPayto.bank(name, ctx))
                         }
                     }
-                } else {
-                    AccountCreationResult.Success(internalPayto.bank(name, ctx))
                 }
+            } else {
+                AccountCreationResult.Success(internalPayto.bank(name, ctx))
             }
         }
     }
@@ -194,21 +197,20 @@ class AccountDAO(private val db: Database) {
     suspend fun delete(
         login: String, 
         is2fa: Boolean
-    ): AccountDeletionResult = db.serializable { conn ->
-        val now = Instant.now().micros()
-        val stmt = conn.prepareStatement("""
-            SELECT
-              out_not_found,
-              out_balance_not_zero,
-              out_tan_required
-              FROM account_delete(?,?,?)
-        """)
-        stmt.setString(1, login)
-        stmt.setLong(2, now)
-        stmt.setBoolean(3, is2fa)
-        stmt.executeQuery().use {
+    ): AccountDeletionResult = db.serializable(
+        """
+        SELECT
+            out_not_found,
+            out_balance_not_zero,
+            out_tan_required
+        FROM account_delete(?,?,?)
+        """
+    ) {
+        setString(1, login)
+        setLong(2, Instant.now().micros())
+        setBoolean(3, is2fa)
+        one {
             when {
-                !it.next() -> throw internalServerError("Deletion returned nothing.")
                 it.getBoolean("out_not_found") -> AccountDeletionResult.UnknownAccount
                 it.getBoolean("out_balance_not_zero") -> AccountDeletionResult.BalanceNotZero
                 it.getBoolean("out_tan_required") -> AccountDeletionResult.TanRequired
@@ -223,6 +225,7 @@ class AccountDAO(private val db: Database) {
         data object NonAdminName: AccountPatchResult
         data object NonAdminCashout: AccountPatchResult
         data object NonAdminDebtLimit: AccountPatchResult
+        data object NonAdminMinCashout: AccountPatchResult
         data object MissingTanInfo: AccountPatchResult
         data class TanRequired(val channel: TanChannel?, val info: String?): AccountPatchResult
         data object Success: AccountPatchResult
@@ -238,16 +241,18 @@ class AccountDAO(private val db: Database) {
         tan_channel: Option<TanChannel?>,
         isPublic: Boolean?,
         debtLimit: TalerAmount?,
+        minCashout: Option<TalerAmount?>,
         isAdmin: Boolean,
         is2fa: Boolean,
         faChannel: TanChannel?,
         faInfo: String?,
         allowEditName: Boolean,
         allowEditCashout: Boolean,
-    ): AccountPatchResult = db.serializable { it.transaction { conn ->
+    ): AccountPatchResult = db.serializableTransaction { conn ->
         val checkName = !isAdmin && !allowEditName && name != null
         val checkCashout = !isAdmin && !allowEditCashout && cashoutPayto.isSome()
         val checkDebtLimit = !isAdmin && debtLimit != null
+        val checkMinCashout = !isAdmin && minCashout.isSome()
 
         data class CurrentAccount(
             val id: Long,
@@ -257,19 +262,22 @@ class AccountDAO(private val db: Database) {
             val name: String,
             val cashoutPayTo: String?,
             val debtLimit: TalerAmount,
+            val minCashout: TalerAmount?
         )
 
         // Get user ID and current data
-        val curr = conn.prepareStatement("""
+        val curr = conn.withStatement("""
             SELECT 
                 customer_id, tan_channel, phone, email, name, cashout_payto
                 ,(max_debt).val AS max_debt_val
                 ,(max_debt).frac AS max_debt_frac
+                ,(min_cashout).val AS min_cashout_val
+                ,(min_cashout).frac AS min_cashout_frac
             FROM customers
                 JOIN bank_accounts 
                 ON customer_id=owning_customer_id
             WHERE login=? AND deleted_at IS NULL
-        """).run {
+        """) {
             setString(1, login)
             oneOrNull {
                 CurrentAccount(
@@ -280,8 +288,9 @@ class AccountDAO(private val db: Database) {
                     name = it.getString("name"),
                     cashoutPayTo = it.getString("cashout_payto"),
                     debtLimit = it.getAmount("max_debt", db.bankCurrency),
+                    minCashout = it.getOptAmount("min_cashout", db.bankCurrency),
                 )
-            } ?: return@transaction AccountPatchResult.UnknownAccount
+            } ?: return@serializableTransaction AccountPatchResult.UnknownAccount
         }
 
         // Patched TAN channel
@@ -305,33 +314,35 @@ class AccountDAO(private val db: Database) {
 
         // Check reconfig rights
         if (checkName && name != curr.name) 
-            return@transaction AccountPatchResult.NonAdminName
+            return@serializableTransaction AccountPatchResult.NonAdminName
         if (checkCashout && fullCashoutPayto != curr.cashoutPayTo) 
-            return@transaction AccountPatchResult.NonAdminCashout
+            return@serializableTransaction AccountPatchResult.NonAdminCashout
         if (checkDebtLimit && debtLimit != curr.debtLimit)
-            return@transaction AccountPatchResult.NonAdminDebtLimit
+            return@serializableTransaction AccountPatchResult.NonAdminDebtLimit
+        if (checkMinCashout && minCashout.get() != curr.minCashout)
+            return@serializableTransaction AccountPatchResult.NonAdminMinCashout
         if (patchChannel != null && newInfo == null)
-            return@transaction AccountPatchResult.MissingTanInfo
-
+            return@serializableTransaction AccountPatchResult.MissingTanInfo
 
         // Tan channel verification
         if (!isAdmin) {
             // Check performed 2fa check
             if (curr.channel != null && !is2fa) {
                 // Perform challenge with current settings
-                return@transaction AccountPatchResult.TanRequired(channel = null, info = null)
+                return@serializableTransaction AccountPatchResult.TanRequired(channel = null, info = null)
             }
             // If channel or info changed and the 2fa challenge is performed with old settings perform a new challenge with new settings
             if ((patchChannel != null && patchChannel != faChannel) || (patchInfo != null && patchInfo != faInfo)) {
-                return@transaction AccountPatchResult.TanRequired(channel = newChannel, info = newInfo)
+                return@serializableTransaction AccountPatchResult.TanRequired(channel = newChannel, info = newInfo)
             }
         }
 
         // Invalidate current challenges
         if (patchChannel != null || patchInfo != null) {
-            val stmt = conn.prepareStatement("UPDATE tan_challenges SET expiration_date=0 WHERE customer=?")
-            stmt.setLong(1, curr.id)
-            stmt.execute()
+            conn.withStatement("UPDATE tan_challenges SET expiration_date=0 WHERE customer=?") {
+                setLong(1, curr.id)
+                execute()
+            }
         }
 
         // Update bank info
@@ -340,11 +351,24 @@ class AccountDAO(private val db: Database) {
             sequence {
                 if (isPublic != null) yield("is_public=?")
                 if (debtLimit != null) yield("max_debt=(?, ?)::taler_amount")
+                minCashout.some { 
+                    if (it != null) {
+                        yield("min_cashout=(?, ?)::taler_amount")
+                    } else {
+                        yield("min_cashout=null")
+                    }
+                }
             },
             "WHERE owning_customer_id = ?",
             sequence {
                 isPublic?.let { yield(it) }
                 debtLimit?.let { yield(it.value); yield(it.frac) }
+                minCashout.some { 
+                    if (it != null) {
+                        yield(it.value)
+                        yield(it.frac)
+                    }
+                }
                 yield(curr.id)
             }
         )
@@ -371,7 +395,7 @@ class AccountDAO(private val db: Database) {
         )
 
         AccountPatchResult.Success
-    }}
+    }
 
 
     /** Result status of customer account auth patch */
@@ -388,59 +412,89 @@ class AccountDAO(private val db: Database) {
         newPw: String, 
         oldPw: String?,
         is2fa: Boolean
-    ): AccountPatchAuthResult = db.serializable {
-        it.transaction { conn ->
-            val (currentPwh, tanRequired) = conn.prepareStatement("""
-                SELECT password_hash, (NOT ? AND tan_channel IS NOT NULL) 
-                FROM customers WHERE login=? AND deleted_at IS NULL
-            """).run {
-                setBoolean(1, is2fa)
-                setString(2, login)
-                oneOrNull { 
-                    Pair(it.getString(1), it.getBoolean(2))
-                } ?: return@transaction AccountPatchAuthResult.UnknownAccount
-            }
-            if (tanRequired) {
-                AccountPatchAuthResult.TanRequired
-            } else if (oldPw != null && !PwCrypto.checkpw(oldPw, currentPwh)) {
-                AccountPatchAuthResult.OldPasswordMismatch
-            } else {
-                val stmt = conn.prepareStatement("""
-                    UPDATE customers SET password_hash=? where login=?
-                """)
-                stmt.setString(1, PwCrypto.hashpw(newPw))
-                stmt.setString(2, login)
-                stmt.executeUpdateCheck()
-                AccountPatchAuthResult.Success
+    ): AccountPatchAuthResult = db.serializableTransaction { conn ->
+        val (customerId, currentPwh, tanRequired) = conn.withStatement("""
+            SELECT customer_id, password_hash, (NOT ? AND tan_channel IS NOT NULL) 
+            FROM customers WHERE login=? AND deleted_at IS NULL
+        """) {
+            setBoolean(1, is2fa)
+            setString(2, login)
+            oneOrNull { 
+                Triple(it.getLong(1), it.getString(2), it.getBoolean(3))
+            } ?: return@serializableTransaction AccountPatchAuthResult.UnknownAccount
+        }
+        if (tanRequired) {
+            AccountPatchAuthResult.TanRequired
+        } else if (oldPw != null && !PwCrypto.checkpw(oldPw, currentPwh).match) {
+            AccountPatchAuthResult.OldPasswordMismatch
+        } else {
+            val newPwh = PwCrypto.hashpw(newPw)
+            conn.withStatement("UPDATE customers SET password_hash=? WHERE customer_id=?") {
+                setString(1, newPwh)
+                setLong(2, customerId)
+                executeUpdate()
             }
+          
+            AccountPatchAuthResult.Success
         }
     }
 
-    /** Get password hash of account [login] */
-    suspend fun passwordHash(login: String): String? = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT password_hash FROM customers WHERE login=? AND deleted_at IS NULL
-        """)
-        stmt.setString(1, login)
-        stmt.oneOrNull { 
-            it.getString(1)
+    /** Result status of customer account password check */
+    enum class CheckPasswordResult {
+        UnknownAccount,
+        PasswordMismatch,
+        Success
+    }
+
+    /** Check password of account [login] against [pw], rehashing it if outdated  */
+    suspend fun checkPassword(login: String, pw: String): CheckPasswordResult {
+        // Get user current password hash
+        val info = db.serializable(
+            "SELECT customer_id, password_hash FROM customers WHERE login=? AND deleted_at IS NULL"
+        ) { 
+            setString(1, login)
+            oneOrNull { 
+                Pair(it.getLong(1), it.getString(2))
+            }
         }
+        if (info == null) return CheckPasswordResult.UnknownAccount
+        val (customerId, currentPwh) = info
+
+        // Check password
+        val check = PwCrypto.checkpw(pw, currentPwh)
+        if (!check.match) return CheckPasswordResult.PasswordMismatch
+
+        // Reshah if outdated
+        if (check.outdated) {
+            val newPwh = PwCrypto.hashpw(pw)
+            db.serializable(
+                "UPDATE customers SET password_hash=? where customer_id=? AND password_hash=?"
+            ) { 
+                setString(1, newPwh)
+                setLong(2, customerId)
+                setString(3, currentPwh)
+                executeUpdate()
+            }
+        }
+
+        return CheckPasswordResult.Success
     }
 
     /** Get bank info of account [login] */
-    suspend fun bankInfo(login: String, ctx: BankPaytoCtx): BankInfo? = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-             bank_account_id
-             ,internal_payto_uri
-             ,name
-             ,is_taler_exchange
-            FROM bank_accounts
-                JOIN customers ON customer_id=owning_customer_id
-            WHERE login=?
-        """)
-        stmt.setString(1, login)
-        stmt.oneOrNull {
+    suspend fun bankInfo(login: String, ctx: BankPaytoCtx): BankInfo? = db.serializable(
+        """
+        SELECT
+            bank_account_id
+            ,internal_payto_uri
+            ,name
+            ,is_taler_exchange
+        FROM bank_accounts
+            JOIN customers ON customer_id=owning_customer_id
+        WHERE login=?
+        """
+    ) {
+        setString(1, login)
+        oneOrNull {
             BankInfo(
                 payto = it.getBankPayto("internal_payto_uri", "name", ctx),
                 isTalerExchange = it.getBoolean("is_taler_exchange"),
@@ -450,33 +504,36 @@ class AccountDAO(private val db: Database) {
     }
 
     /** Get data of account [login] */
-    suspend fun get(login: String, ctx: BankPaytoCtx): AccountData? = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-                name
-                ,email
-                ,phone
-                ,tan_channel
-                ,cashout_payto
-                ,internal_payto_uri
-                ,(balance).val AS balance_val
-                ,(balance).frac AS balance_frac
-                ,has_debt
-                ,(max_debt).val AS max_debt_val
-                ,(max_debt).frac AS max_debt_frac
-                ,is_public
-                ,is_taler_exchange
-                ,CASE 
-                    WHEN deleted_at IS NOT NULL THEN 'deleted'
-                    ELSE 'active'
-                END as status
-            FROM customers 
-                JOIN bank_accounts
-                    ON customer_id=owning_customer_id
-            WHERE login=?
-        """)
-        stmt.setString(1, login)
-        stmt.oneOrNull {
+    suspend fun get(login: String, ctx: BankPaytoCtx): AccountData? = db.serializable(
+        """
+        SELECT
+            name
+            ,email
+            ,phone
+            ,tan_channel
+            ,cashout_payto
+            ,internal_payto_uri
+            ,(balance).val AS balance_val
+            ,(balance).frac AS balance_frac
+            ,has_debt
+            ,(max_debt).val AS max_debt_val
+            ,(max_debt).frac AS max_debt_frac
+            ,(min_cashout).val AS min_cashout_val
+            ,(min_cashout).frac AS min_cashout_frac
+            ,is_public
+            ,is_taler_exchange
+            ,CASE 
+                WHEN deleted_at IS NOT NULL THEN 'deleted'
+                ELSE 'active'
+            END as status
+        FROM customers 
+            JOIN bank_accounts
+                ON customer_id=owning_customer_id
+        WHERE login=?
+        """
+    ) {
+        setString(1, login)
+        oneOrNull {
             AccountData(
                 name = it.getString("name"),
                 contact_data = ChallengeContactData(
@@ -496,6 +553,7 @@ class AccountDAO(private val db: Database) {
                         }
                 ),
                 debit_threshold = it.getAmount("max_debt", db.bankCurrency),
+                min_cashout = it.getOptAmount("min_cashout", db.bankCurrency),
                 is_public = it.getBoolean("is_public"),
                 is_taler_exchange = it.getBoolean("is_taler_exchange"),
                 status = AccountStatus.valueOf(it.getString("status"))
@@ -504,7 +562,7 @@ class AccountDAO(private val db: Database) {
     }
 
     /** Get a page of all public accounts */
-    suspend fun pagePublic(params: AccountParams, ctx: BankPaytoCtx): List<PublicAccount>
+    suspend fun pagePublic(params: AccountParams, ctx: BankPaytoCtx): List<PublicAccount> 
         = db.page(
             params.page,
             "bank_account_id",
@@ -520,11 +578,17 @@ class AccountDAO(private val db: Database) {
               bank_account_id
               FROM bank_accounts JOIN customers
                 ON owning_customer_id = customer_id 
-              WHERE is_public=true AND name LIKE ? AND deleted_at IS NULL AND
+              WHERE is_public=true AND 
+                ${if (params.loginFilter != null) "name LIKE ? AND" else ""} 
+                deleted_at IS NULL AND
             """,
             {
-                setString(1, params.loginFilter)
-                1
+                if (params.loginFilter != null) {
+                    setString(1, params.loginFilter)
+                    1
+                } else {
+                    0
+                }
             }
         ) {
             PublicAccount(
@@ -557,6 +621,8 @@ class AccountDAO(private val db: Database) {
             has_debt AS balance_has_debt,
             (max_debt).val as max_debt_val,
             (max_debt).frac as max_debt_frac
+            ,(min_cashout).val AS min_cashout_val
+            ,(min_cashout).frac AS min_cashout_frac
             ,is_public
             ,is_taler_exchange
             ,internal_payto_uri
@@ -567,11 +633,15 @@ class AccountDAO(private val db: Database) {
             END as status
             FROM bank_accounts JOIN customers
               ON owning_customer_id = customer_id 
-            WHERE name LIKE ? AND
+            WHERE ${if (params.loginFilter != null) "name LIKE ? AND" else ""} 
             """,
             {
-                setString(1, params.loginFilter)
-                1
+                if (params.loginFilter != null) {
+                    setString(1, params.loginFilter)
+                    1
+                } else {
+                    0
+                }
             }
         ) {
             AccountMinimalData(
@@ -587,6 +657,7 @@ class AccountDAO(private val db: Database) {
                     }
                 ),
                 debit_threshold = it.getAmount("max_debt", db.bankCurrency),
+                min_cashout = it.getOptAmount("min_cashout", db.bankCurrency),
                 is_public = it.getBoolean("is_public"),
                 is_taler_exchange = it.getBoolean("is_taler_exchange"),
                 payto_uri = it.getBankPayto("internal_payto_uri", "name", ctx),
