@@ -1,6 +1,6 @@
 /*
  * This file is part of LibEuFin.
- * Copyright (C) 2023 Taler Systems S.A.
+ * Copyright (C) 2023-2024 Taler Systems S.A.
 
  * LibEuFin is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -46,35 +46,34 @@ class CashoutDAO(private val db: Database) {
         amountDebit: TalerAmount,
         amountCredit: TalerAmount,
         subject: String,
-        now: Instant,
+        timestamp: Instant,
         is2fa: Boolean
-    ): CashoutCreationResult = db.serializable { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-                out_bad_conversion,
-                out_account_not_found,
-                out_account_is_exchange,
-                out_balance_insufficient,
-                out_request_uid_reuse,
-                out_no_cashout_payto,
-                out_tan_required,
-                out_cashout_id,
-                out_under_min
-            FROM cashout_create(?,?,(?,?)::taler_amount,(?,?)::taler_amount,?,?,?)
-        """)
-        stmt.setString(1, login)
-        stmt.setBytes(2, requestUid.raw)
-        stmt.setLong(3, amountDebit.value)
-        stmt.setInt(4, amountDebit.frac)
-        stmt.setLong(5, amountCredit.value)
-        stmt.setInt(6, amountCredit.frac)
-        stmt.setString(7, subject)
-        stmt.setLong(8, now.micros())
-        stmt.setBoolean(9, is2fa)
-        stmt.executeQuery().use {
+    ): CashoutCreationResult = db.serializable(
+        """
+        SELECT
+            out_bad_conversion,
+            out_account_not_found,
+            out_account_is_exchange,
+            out_balance_insufficient,
+            out_request_uid_reuse,
+            out_no_cashout_payto,
+            out_tan_required,
+            out_cashout_id,
+            out_under_min
+        FROM cashout_create(?,?,(?,?)::taler_amount,(?,?)::taler_amount,?,?,?)
+        """
+    ) {
+        setString(1, login)
+        setBytes(2, requestUid.raw)
+        setLong(3, amountDebit.value)
+        setInt(4, amountDebit.frac)
+        setLong(5, amountCredit.value)
+        setInt(6, amountCredit.frac)
+        setString(7, subject)
+        setLong(8, timestamp.micros())
+        setBoolean(9, is2fa)
+        one {
             when {
-                !it.next() ->
-                    throw internalServerError("No result from DB procedure cashout_create")
                 it.getBoolean("out_under_min") -> CashoutCreationResult.UnderMin
                 it.getBoolean("out_bad_conversion") -> CashoutCreationResult.BadConversion
                 it.getBoolean("out_account_not_found") -> CashoutCreationResult.AccountNotFound
@@ -89,30 +88,31 @@ class CashoutDAO(private val db: Database) {
     }
 
     /** Get status of cashout operation [id] owned by [login] */
-    suspend fun get(id: Long, login: String): CashoutStatusResponse? = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-                (amount_debit).val as amount_debit_val
-                ,(amount_debit).frac as amount_debit_frac
-                ,(amount_credit).val as amount_credit_val
-                ,(amount_credit).frac as amount_credit_frac
-                ,cashout_operations.subject
-                ,creation_time
-                ,transaction_date as confirmation_date
-                ,tan_channel
-                ,CASE tan_channel
-                    WHEN 'sms'   THEN phone
-                    WHEN 'email' THEN email
-                END as tan_info
-            FROM cashout_operations
-                JOIN bank_accounts ON bank_account=bank_account_id
-                JOIN customers ON owning_customer_id=customer_id
-                LEFT JOIN bank_account_transactions ON local_transaction=bank_transaction_id
-            WHERE cashout_id=? AND login=?
-        """)
-        stmt.setLong(1, id)
-        stmt.setString(2, login)
-        stmt.oneOrNull {
+    suspend fun get(id: Long, login: String): CashoutStatusResponse? = db.serializable(
+        """
+        SELECT
+            (amount_debit).val as amount_debit_val
+            ,(amount_debit).frac as amount_debit_frac
+            ,(amount_credit).val as amount_credit_val
+            ,(amount_credit).frac as amount_credit_frac
+            ,cashout_operations.subject
+            ,creation_time
+            ,transaction_date as confirmation_date
+            ,tan_channel
+            ,CASE tan_channel
+                WHEN 'sms'   THEN phone
+                WHEN 'email' THEN email
+            END as tan_info
+        FROM cashout_operations
+            JOIN bank_accounts ON bank_account=bank_account_id
+            JOIN customers ON owning_customer_id=customer_id
+            LEFT JOIN bank_account_transactions ON local_transaction=bank_transaction_id
+        WHERE cashout_id=? AND login=?
+        """
+    ) {
+        setLong(1, id)
+        setString(2, login)
+        oneOrNull {
             CashoutStatusResponse(
                 status = CashoutStatus.confirmed,
                 amount_debit = it.getAmount("amount_debit", db.bankCurrency),
@@ -132,9 +132,7 @@ class CashoutDAO(private val db: Database) {
     /** Get a page of all cashout operations */
     suspend fun pageAll(params: PageParams): List<GlobalCashoutInfo> =
         db.page(params, "cashout_id", """
-            SELECT
-                cashout_id
-                ,login
+            SELECT cashout_id, login
             FROM cashout_operations
                 JOIN bank_accounts ON bank_account=bank_account_id
                 JOIN customers ON owning_customer_id=customer_id
@@ -152,9 +150,11 @@ class CashoutDAO(private val db: Database) {
         db.page(params, "cashout_id", """
             SELECT cashout_id
             FROM cashout_operations
-                JOIN bank_accounts ON bank_account=bank_account_id
-                JOIN customers ON owning_customer_id=customer_id
-            WHERE login = ? AND
+            WHERE bank_account=(
+                SELECT bank_account_id 
+                FROM bank_accounts JOIN customers ON owning_customer_id=customer_id 
+                WHERE deleted_at IS NULL AND login = ?
+            ) AND 
         """, 
             bind = { 
                 setString(1, login)
