@@ -19,8 +19,8 @@
 
 package tech.libeufin.bank.db
 
-import tech.libeufin.bank.*
 import tech.libeufin.common.*
+import tech.libeufin.common.db.*
 import java.time.Instant
 
 /** Data access logic for exchange specific logic */
@@ -31,7 +31,7 @@ class ExchangeDAO(private val db: Database) {
         exchangeId: Long,
         ctx: BankPaytoCtx
     ): List<IncomingReserveTransaction> 
-        = db.poolHistory(params, exchangeId, NotificationWatcher::listenIncoming,  """
+        = db.poolHistory(params, exchangeId, db::listenIncoming,  """
             SELECT
                 bank_transaction_id
                 ,transaction_date
@@ -60,7 +60,7 @@ class ExchangeDAO(private val db: Database) {
         exchangeId: Long,
         ctx: BankPaytoCtx
     ): List<OutgoingTransaction> 
-        = db.poolHistory(params, exchangeId, NotificationWatcher::listenOutgoing,  """
+        = db.poolHistory(params, exchangeId, db::listenOutgoing,  """
             SELECT
                 bank_transaction_id
                 ,transaction_date
@@ -101,41 +101,40 @@ class ExchangeDAO(private val db: Database) {
     suspend fun transfer(
         req: TransferRequest,
         login: String,
-        now: Instant
-    ): TransferResult = db.serializable { conn ->
+        timestamp: Instant
+    ): TransferResult = db.serializable(
+        """
+        SELECT
+            out_debtor_not_found
+            ,out_debtor_not_exchange
+            ,out_creditor_not_found
+            ,out_both_exchanges
+            ,out_request_uid_reuse
+            ,out_exchange_balance_insufficient
+            ,out_tx_row_id
+            ,out_timestamp
+        FROM
+            taler_transfer (
+                ?, ?, ?,
+                (?,?)::taler_amount,
+                ?, ?, ?, ?
+            );
+        """
+    ) {
         val subject = "${req.wtid} ${req.exchange_base_url.url}"
-        val stmt = conn.prepareStatement("""
-            SELECT
-                out_debtor_not_found
-                ,out_debtor_not_exchange
-                ,out_creditor_not_found
-                ,out_both_exchanges
-                ,out_request_uid_reuse
-                ,out_exchange_balance_insufficient
-                ,out_tx_row_id
-                ,out_timestamp
-              FROM
-              taler_transfer (
-                  ?, ?, ?,
-                  (?,?)::taler_amount,
-                  ?, ?, ?, ?
-                );
-        """)
 
-        stmt.setBytes(1, req.request_uid.raw)
-        stmt.setBytes(2, req.wtid.raw)
-        stmt.setString(3, subject)
-        stmt.setLong(4, req.amount.value)
-        stmt.setInt(5, req.amount.frac)
-        stmt.setString(6, req.exchange_base_url.url)
-        stmt.setString(7, req.credit_account.canonical)
-        stmt.setString(8, login)
-        stmt.setLong(9, now.toDbMicros() ?: throw faultyTimestampByBank())
+        setBytes(1, req.request_uid.raw)
+        setBytes(2, req.wtid.raw)
+        setString(3, subject)
+        setLong(4, req.amount.value)
+        setInt(5, req.amount.frac)
+        setString(6, req.exchange_base_url.url)
+        setString(7, req.credit_account.canonical)
+        setString(8, login)
+        setLong(9, timestamp.micros())
 
-        stmt.executeQuery().use {
+        one {
             when {
-                !it.next() ->
-                    throw internalServerError("SQL function taler_transfer did not return anything.")
                 it.getBoolean("out_debtor_not_found") -> TransferResult.UnknownExchange
                 it.getBoolean("out_debtor_not_exchange") -> TransferResult.NotAnExchange
                 it.getBoolean("out_creditor_not_found") -> TransferResult.UnknownCreditor
@@ -166,37 +165,35 @@ class ExchangeDAO(private val db: Database) {
     suspend fun addIncoming(
         req: AddIncomingRequest,
         login: String,
-        now: Instant
-    ): AddIncomingResult = db.serializable { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-                out_creditor_not_found
-                ,out_creditor_not_exchange
-                ,out_debtor_not_found
-                ,out_both_exchanges
-                ,out_reserve_pub_reuse
-                ,out_debitor_balance_insufficient
-                ,out_tx_row_id
-            FROM
+        timestamp: Instant
+    ): AddIncomingResult = db.serializable(
+        """
+        SELECT
+            out_creditor_not_found
+            ,out_creditor_not_exchange
+            ,out_debtor_not_found
+            ,out_both_exchanges
+            ,out_reserve_pub_reuse
+            ,out_debitor_balance_insufficient
+            ,out_tx_row_id
+        FROM
             taler_add_incoming (
                 ?, ?,
                 (?,?)::taler_amount,
                 ?, ?, ?
-                );
-        """)
+            );
+        """
+    ) {
+        setBytes(1, req.reserve_pub.raw)
+        setString(2, "Manual incoming ${req.reserve_pub}")
+        setLong(3, req.amount.value)
+        setInt(4, req.amount.frac)
+        setString(5, req.debit_account.canonical)
+        setString(6, login)
+        setLong(7, timestamp.micros())
 
-        stmt.setBytes(1, req.reserve_pub.raw)
-        stmt.setString(2, "Manual incoming ${req.reserve_pub}")
-        stmt.setLong(3, req.amount.value)
-        stmt.setInt(4, req.amount.frac)
-        stmt.setString(5, req.debit_account.canonical)
-        stmt.setString(6, login)
-        stmt.setLong(7, now.toDbMicros() ?: throw faultyTimestampByBank())
-
-        stmt.executeQuery().use {
+        one {
             when {
-                !it.next() ->
-                    throw internalServerError("SQL function taler_add_incoming did not return anything.")
                 it.getBoolean("out_creditor_not_found") -> AddIncomingResult.UnknownExchange 
                 it.getBoolean("out_creditor_not_exchange") -> AddIncomingResult.NotAnExchange
                 it.getBoolean("out_debtor_not_found") -> AddIncomingResult.UnknownDebtor
@@ -205,7 +202,7 @@ class ExchangeDAO(private val db: Database) {
                 it.getBoolean("out_reserve_pub_reuse") -> AddIncomingResult.ReservePubReuse
                 else -> AddIncomingResult.Success(
                     id = it.getLong("out_tx_row_id"),
-                    timestamp = TalerProtocolTimestamp(now)
+                    timestamp = TalerProtocolTimestamp(timestamp)
                 )
             }
         }
