@@ -21,8 +21,10 @@ package tech.libeufin.bank.db
 
 import org.slf4j.Logger
 import org.slf4j.LoggerFactory
-import tech.libeufin.bank.*
+import tech.libeufin.bank.BankAccountTransactionInfo
+import tech.libeufin.bank.TransactionDirection
 import tech.libeufin.common.*
+import tech.libeufin.common.db.*
 import java.time.Instant
 
 private val logger: Logger = LoggerFactory.getLogger("libeufin-bank-tx-dao")
@@ -37,7 +39,9 @@ class TransactionDAO(private val db: Database) {
         data object UnknownDebtor: BankTransactionResult
         data object BothPartySame: BankTransactionResult
         data object BalanceInsufficient: BankTransactionResult
+        data object BadAmount: BankTransactionResult
         data object TanRequired: BankTransactionResult
+        data object RequestUidReuse: BankTransactionResult
     }
 
     /** Create a new transaction */
@@ -47,42 +51,57 @@ class TransactionDAO(private val db: Database) {
         subject: String,
         amount: TalerAmount,
         timestamp: Instant,
-        is2fa: Boolean
-    ): BankTransactionResult = db.serializable { conn ->
-        val now = timestamp.toDbMicros() ?: throw faultyTimestampByBank()
-        conn.transaction {
-            val stmt = conn.prepareStatement("""
-                SELECT 
-                    out_creditor_not_found 
-                    ,out_debtor_not_found
-                    ,out_same_account
-                    ,out_balance_insufficient
-                    ,out_tan_required
-                    ,out_credit_bank_account_id
-                    ,out_debit_bank_account_id
-                    ,out_credit_row_id
-                    ,out_debit_row_id
-                    ,out_creditor_is_exchange 
-                    ,out_debtor_is_exchange
-                    ,out_creditor_admin
-                FROM bank_transaction(?,?,?,(?,?)::taler_amount,?,?)
-            """
-            )
-            stmt.setString(1, creditAccountPayto.canonical)
-            stmt.setString(2, debitAccountUsername)
-            stmt.setString(3, subject)
-            stmt.setLong(4, amount.value)
-            stmt.setInt(5, amount.frac)
-            stmt.setLong(6, now)
-            stmt.setBoolean(7, is2fa)
-            stmt.executeQuery().use {
+        is2fa: Boolean,
+        requestUid: ShortHashCode?,
+        wireTransferFees: TalerAmount,
+        minAmount: TalerAmount,
+        maxAmount: TalerAmount
+    ): BankTransactionResult = db.serializableTransaction { conn ->
+        val timestamp = timestamp.micros()
+        conn.withStatement("""
+            SELECT 
+                out_creditor_not_found 
+                ,out_debtor_not_found
+                ,out_same_account
+                ,out_balance_insufficient
+                ,out_bad_amount
+                ,out_request_uid_reuse
+                ,out_tan_required
+                ,out_credit_bank_account_id
+                ,out_debit_bank_account_id
+                ,out_credit_row_id
+                ,out_debit_row_id
+                ,out_creditor_is_exchange 
+                ,out_debtor_is_exchange
+                ,out_creditor_admin
+                ,out_idempotent
+            FROM bank_transaction(?,?,?,(?,?)::taler_amount,?,?,?,(?,?)::taler_amount,(?,?)::taler_amount,(?,?)::taler_amount)
+        """
+        ) {
+            setString(1, creditAccountPayto.canonical)
+            setString(2, debitAccountUsername)
+            setString(3, subject)
+            setLong(4, amount.value)
+            setInt(5, amount.frac)
+            setLong(6, timestamp)
+            setBoolean(7, is2fa)
+            setBytes(8, requestUid?.raw)
+            setLong(9, wireTransferFees.value)
+            setInt(10, wireTransferFees.frac)
+            setLong(11, minAmount.value)
+            setInt(12, minAmount.frac)
+            setLong(13, maxAmount.value)
+            setInt(14, maxAmount.frac)
+            one {
                 when {
-                    !it.next() -> throw internalServerError("Bank transaction didn't properly return")
                     it.getBoolean("out_creditor_not_found") -> BankTransactionResult.UnknownCreditor
                     it.getBoolean("out_debtor_not_found") -> BankTransactionResult.UnknownDebtor
                     it.getBoolean("out_same_account") -> BankTransactionResult.BothPartySame
                     it.getBoolean("out_balance_insufficient") -> BankTransactionResult.BalanceInsufficient
+                    it.getBoolean("out_bad_amount") -> BankTransactionResult.BadAmount
                     it.getBoolean("out_creditor_admin") -> BankTransactionResult.AdminCreditor
+                    it.getBoolean("out_request_uid_reuse") -> BankTransactionResult.RequestUidReuse
+                    it.getBoolean("out_idempotent") -> BankTransactionResult.Success(it.getLong("out_debit_row_id"))
                     it.getBoolean("out_tan_required") -> BankTransactionResult.TanRequired
                     else -> {
                         val creditAccountId = it.getLong("out_credit_bank_account_id")
@@ -96,7 +115,7 @@ class TransactionDAO(private val db: Database) {
                         } else if (exchangeCreditor) {
                             val bounceCause = runCatching { parseIncomingTxMetadata(subject) }.fold(
                                 onSuccess = { reservePub -> 
-                                    val registered = conn.prepareStatement("CALL register_incoming(?, ?)").run {
+                                    val registered = conn.withStatement("CALL register_incoming(?, ?)") {
                                         setBytes(1, reservePub.raw)
                                         setLong(2, creditRowId)
                                         executeProcedureViolation()
@@ -115,19 +134,17 @@ class TransactionDAO(private val db: Database) {
                             )
                             if (bounceCause != null) {
                                 // No error can happens because an opposite transaction already took place in the same transaction
-                                conn.prepareStatement("""
+                                conn.withStatement("""
                                     SELECT bank_wire_transfer(
-                                        ?, ?, ?, (?, ?)::taler_amount, ?,
-                                        NULL, NULL, NULL
+                                        ?, ?, ?, (?, ?)::taler_amount, ?, NULL,NULL,NULL
                                     );
-                                """
-                                ).run {
+                                """) {
                                     setLong(1, debitAccountId)
                                     setLong(2, creditAccountId)
                                     setString(3, "Bounce $creditRowId: $bounceCause")
                                     setLong(4, amount.value)
                                     setInt(5, amount.frac)
-                                    setLong(6, now)
+                                    setLong(6, timestamp)
                                     executeQuery()
                                 }
                             }
@@ -142,27 +159,28 @@ class TransactionDAO(private val db: Database) {
     }
     
     /** Get transaction [rowId] owned by [login] */
-    suspend fun get(rowId: Long, login: String, ctx: BankPaytoCtx): BankAccountTransactionInfo? = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT 
-              creditor_payto_uri
-              ,creditor_name
-              ,debtor_payto_uri
-              ,debtor_name
-              ,subject
-              ,(amount).val AS amount_val
-              ,(amount).frac AS amount_frac
-              ,transaction_date
-              ,direction
-              ,bank_transaction_id
-            FROM bank_account_transactions
-                JOIN bank_accounts ON bank_account_transactions.bank_account_id=bank_accounts.bank_account_id
-                JOIN customers ON customer_id=owning_customer_id 
-	        WHERE bank_transaction_id=? AND login=?
-        """)
-        stmt.setLong(1, rowId)
-        stmt.setString(2, login)
-        stmt.oneOrNull {
+    suspend fun get(rowId: Long, login: String, ctx: BankPaytoCtx): BankAccountTransactionInfo? = db.serializable(
+        """
+        SELECT 
+            creditor_payto_uri
+            ,creditor_name
+            ,debtor_payto_uri
+            ,debtor_name
+            ,subject
+            ,(amount).val AS amount_val
+            ,(amount).frac AS amount_frac
+            ,transaction_date
+            ,direction
+            ,bank_transaction_id
+        FROM bank_account_transactions
+            JOIN bank_accounts ON bank_account_transactions.bank_account_id=bank_accounts.bank_account_id
+            JOIN customers ON customer_id=owning_customer_id 
+        WHERE bank_transaction_id=? AND login=?
+        """
+    ) {
+        setLong(1, rowId)
+        setString(2, login)
+        oneOrNull {
             BankAccountTransactionInfo(
                 creditor_payto_uri = it.getBankPayto("creditor_payto_uri", "creditor_name", ctx),
                 debtor_payto_uri = it.getBankPayto("debtor_payto_uri", "debtor_name", ctx),
@@ -181,7 +199,7 @@ class TransactionDAO(private val db: Database) {
         accountId: Long,
         ctx: BankPaytoCtx
     ): List<BankAccountTransactionInfo> {
-        return db.poolHistory(params, accountId, NotificationWatcher::listenBank,  """
+        return db.poolHistory(params, accountId, db::listenBank,  """
             SELECT
                 bank_transaction_id
                 ,transaction_date
@@ -213,7 +231,7 @@ class TransactionDAO(private val db: Database) {
         accountId: Long,
         ctx: BankPaytoCtx
     ): List<RevenueIncomingBankTransaction> 
-        = db.poolHistory(params, accountId, NotificationWatcher::listenRevenue, """
+        = db.poolHistory(params, accountId, db::listenRevenue, """
             SELECT
                 bank_transaction_id
                 ,transaction_date
