@@ -1,6 +1,6 @@
 /*
  * This file is part of LibEuFin.
- * Copyright (C) 2023 Taler Systems S.A.
+ * Copyright (C) 2023-2024 Taler Systems S.A.
 
  * LibEuFin is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -25,7 +25,6 @@ import io.ktor.server.plugins.*
 import io.ktor.server.request.*
 import io.ktor.server.routing.*
 import io.ktor.server.util.*
-import io.ktor.util.pipeline.*
 import kotlinx.serialization.KSerializer
 import kotlinx.serialization.Serializable
 import kotlinx.serialization.descriptors.PrimitiveKind
@@ -37,7 +36,7 @@ import tech.libeufin.bank.auth.username
 import tech.libeufin.bank.db.AccountDAO.AccountCreationResult
 import tech.libeufin.bank.db.Database
 import tech.libeufin.common.*
-import tech.libeufin.common.api.*
+import tech.libeufin.common.api.intercept
 import java.util.*
 
 fun ApplicationCall.uuidPath(name: String): UUID {
@@ -77,7 +76,7 @@ fun ApplicationRequest.talerWithdrawUri(id: UUID) = url {
     )
     host = "withdraw"
     appendPathSegments("${origin.serverHost}:${origin.serverPort}")
-    headers["X-Forward-Prefix"]?.let {
+    headers[X_FORWARD_PREFIX]?.let {
         appendPathSegments(it)
     }
     appendPathSegments("taler-integration", id.toString())
@@ -89,7 +88,7 @@ fun ApplicationRequest.withdrawConfirmUrl(id: UUID) = url {
         defaultPort = -1
     )
     host = "${origin.serverHost}:${origin.serverPort}"
-    headers["X-Forward-Prefix"]?.let {
+    headers[X_FORWARD_PREFIX]?.let {
         appendPathSegments(it)
     }
     appendEncodedPathSegments("webui", "#", "operation", id.toString())
@@ -105,8 +104,7 @@ fun ApplicationRequest.withdrawConfirmUrl(id: UUID) = url {
 suspend fun createAdminAccount(db: Database, cfg: BankConfig, pw: String? = null): AccountCreationResult {
     var pwStr = pw
     if (pwStr == null) {
-        val pwBuf = ByteArray(32)
-        Random().nextBytes(pwBuf)
+        val pwBuf = ByteArray(32).rand()
         pwStr = String(pwBuf, Charsets.UTF_8)
     }
     
