@@ -25,19 +25,18 @@ import java.time.Instant
 import java.time.LocalDateTime
 import java.time.ZoneOffset
 import java.time.temporal.TemporalAdjusters
-import java.util.*
 
 data class MonitorParams(
     val timeframe: Timeframe,
     val date: LocalDateTime
 ) {
-    constructor(timeframe: Timeframe, now: LocalDateTime, which: Int) : this(
+    constructor(timeframe: Timeframe, timestamp: LocalDateTime, which: Int) : this(
         timeframe,
         when (timeframe) {
-            Timeframe.hour -> now.withHour(which)
-            Timeframe.day -> now.withDayOfMonth(which)
-            Timeframe.month -> now.withMonth(which)
-            Timeframe.year -> now.withYear(which)
+            Timeframe.hour -> timestamp.withHour(which)
+            Timeframe.day -> timestamp.withDayOfMonth(which)
+            Timeframe.month -> timestamp.withMonth(which)
+            Timeframe.year -> timestamp.withYear(which)
         }
     )
     constructor(timeframe: Timeframe, secs: Long) : this(
@@ -49,7 +48,7 @@ data class MonitorParams(
         val names_fmt = names.joinToString()
         
         fun extract(params: Parameters): MonitorParams {
-            val raw = params.get("timeframe") ?: "hour"
+            val raw = params["timeframe"] ?: "hour"
             if (!names.contains(raw)) {
                 throw badRequest("Param 'timeframe' must be one of $names_fmt", TalerErrorCode.GENERIC_PARAMETER_MALFORMED)
             }
@@ -84,11 +83,11 @@ data class MonitorParams(
 }
 
 data class AccountParams(
-    val page: PageParams, val loginFilter: String
+    val page: PageParams, val loginFilter: String?
 ) {
     companion object {
         fun extract(params: Parameters): AccountParams {
-            val loginFilter = params.get("filter_name")?.run { "%$this%" } ?: "%"
+            val loginFilter = params["filter_name"]?.run { "%$this%" }
             return AccountParams(PageParams.extract(params), loginFilter)
         }
     }
@@ -116,10 +115,10 @@ data class StatusParams(
     val old_state: WithdrawalStatus
 ) {
     companion object {
-        val names = WithdrawalStatus.entries.map { it.name }
-        val names_fmt = names.joinToString()
+        private val names = WithdrawalStatus.entries.map { it.name }
+        private val names_fmt = names.joinToString()
         fun extract(params: Parameters): StatusParams {
-            val old_state = params.get("old_state") ?: "pending"
+            val old_state = params["old_state"] ?: "pending"
             if (!names.contains(old_state)) {
                 throw badRequest("Param 'old_state' must be one of $names_fmt", TalerErrorCode.GENERIC_PARAMETER_MALFORMED)
             }
