@@ -21,7 +21,7 @@ import org.junit.Test
 import tech.libeufin.bank.db.TransactionDAO.BankTransactionResult
 import tech.libeufin.bank.db.WithdrawalDAO.WithdrawalCreationResult
 import tech.libeufin.common.*
-import tech.libeufin.common.db.oneOrNull
+import tech.libeufin.common.db.one
 import java.time.Instant
 import java.util.*
 import kotlin.test.assertEquals
@@ -54,7 +54,10 @@ class AmountTest {
                 amount = due,
                 timestamp = Instant.now(),
                 is2fa = false,
-                requestUid = null
+                requestUid = null,
+                wireTransferFees = TalerAmount.zero("KUDOS"),
+                minAmount = TalerAmount.zero("KUDOS"),
+                maxAmount = TalerAmount.max("KUDOS")
             )
             val txBool = when (txRes) {
                 BankTransactionResult.BalanceInsufficient -> false
@@ -64,20 +67,26 @@ class AmountTest {
 
             // Check whithdraw 
             stmt.executeUpdate()
-            val wRes = db.withdrawal.create(
-                login = "merchant",
-                uuid = UUID.randomUUID(),
-                amount = due,
-                now = Instant.now()
-            )
-            val wBool = when (wRes) {
-                WithdrawalCreationResult.BalanceInsufficient -> false
-                WithdrawalCreationResult.Success -> true
-                else -> throw Exception("Unexpected error $txRes")
+            for ((amount, suggested) in listOf(Pair(due, null), Pair(null, due), Pair(due, due))) {
+                val wRes = db.withdrawal.create(
+                    login = "merchant",
+                    uuid = UUID.randomUUID(),
+                    amount = due,
+                    suggested_amount = null,
+                    timestamp = Instant.now(),
+                    wireTransferFees = TalerAmount.zero("KUDOS"),
+                    minAmount = TalerAmount.zero("KUDOS"),
+                    maxAmount = TalerAmount.max("KUDOS")
+                )
+                val wBool = when (wRes) {
+                    WithdrawalCreationResult.BalanceInsufficient -> false
+                    WithdrawalCreationResult.Success -> true
+                    else -> throw Exception("Unexpected error $txRes")
+                }
+                // Logic must be the same
+                assertEquals(wBool, txBool)
             }
-
-            // Logic must be the same
-            assertEquals(wBool, txBool)
+            
             return txBool
         }
 
@@ -136,16 +145,16 @@ class AmountTest {
     fun normalize() = dbSetup { db ->
         db.conn { conn ->
             val stmt = conn.prepareStatement("SELECT normalized.val, normalized.frac FROM amount_normalize((?, ?)::taler_amount) as normalized")
-            fun TalerAmount.normalize(): TalerAmount? {
+            fun TalerAmount.normalize(): TalerAmount {
                 stmt.setLong(1, value)
                 stmt.setInt(2, frac)
-                return stmt.oneOrNull {
+                return stmt.one {
                     TalerAmount(
                         it.getLong(1),
                         it.getInt(2),
                         "EUR"
                     )
-                }!!
+                }
             }
     
             assertEquals(TalerAmount("EUR:6"), TalerAmount(4L, 2 * TalerAmount.FRACTION_BASE, "EUR").normalize())
@@ -153,6 +162,10 @@ class AmountTest {
             assertEquals(TalerAmount("EUR:${TalerAmount.MAX_VALUE}.99999999"), TalerAmount("EUR:${TalerAmount.MAX_VALUE}.99999999").normalize())
             assertException("ERROR: bigint out of range") { TalerAmount(Long.MAX_VALUE, TalerAmount.FRACTION_BASE, "EUR").normalize() }
             assertException("ERROR: amount value overflowed") { TalerAmount(TalerAmount.MAX_VALUE, TalerAmount.FRACTION_BASE , "EUR").normalize() }
+
+            for (amount in listOf(TalerAmount.max("EUR"), TalerAmount.zero("EUR"))) {
+                assertEquals(amount, amount.normalize())
+            }
         }
     }
 
@@ -160,20 +173,21 @@ class AmountTest {
     fun add() = dbSetup { db ->
         db.conn { conn ->
             val stmt = conn.prepareStatement("SELECT sum.val, sum.frac FROM amount_add((?, ?)::taler_amount, (?, ?)::taler_amount) as sum")
-            operator fun TalerAmount.plus(increment: TalerAmount): TalerAmount? {
+            operator fun TalerAmount.plus(increment: TalerAmount): TalerAmount {
                 stmt.setLong(1, value)
                 stmt.setInt(2, frac)
                 stmt.setLong(3, increment.value)
                 stmt.setInt(4, increment.frac)
-                return stmt.oneOrNull {
+                return stmt.one {
                     TalerAmount(
                         it.getLong(1),
                         it.getInt(2),
                         "EUR"
                     )
-                }!!
+                }
             }
-    
+            assertEquals(TalerAmount.max("EUR"), TalerAmount.max("EUR") + TalerAmount.zero("EUR"))
+            assertEquals(TalerAmount.zero("EUR"), TalerAmount.zero("EUR") + TalerAmount.zero("EUR"))
             assertEquals(TalerAmount("EUR:6.41") + TalerAmount("EUR:4.69"), TalerAmount("EUR:11.1"))
             assertEquals(TalerAmount("EUR:${TalerAmount.MAX_VALUE}") + TalerAmount("EUR:0.99999999"), TalerAmount("EUR:${TalerAmount.MAX_VALUE}.99999999"))
             assertException("ERROR: amount value overflowed") { TalerAmount(TalerAmount.MAX_VALUE - 5, 0, "EUR") + TalerAmount(6, 0, "EUR") }
@@ -186,7 +200,7 @@ class AmountTest {
     @Test
     fun conversionApply() = dbSetup { db ->
         db.conn { conn ->
-            fun apply(nb: TalerAmount, times: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount? {
+            fun apply(nb: TalerAmount, times: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount {
                 val stmt = conn.prepareStatement("SELECT amount.val, amount.frac FROM conversion_apply_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode) as amount")
                 stmt.setLong(1, nb.value)
                 stmt.setInt(2, nb.frac)
@@ -195,13 +209,13 @@ class AmountTest {
                 stmt.setLong(5, tiny.value)
                 stmt.setInt(6, tiny.frac)
                 stmt.setString(7, roundingMode)
-                return stmt.oneOrNull {
+                return stmt.one {
                     TalerAmount(
                         it.getLong(1),
                         it.getInt(2),
                         nb.currency
                     )
-                }!!
+                }
             }
     
             assertEquals(TalerAmount("EUR:30.0629"), apply(TalerAmount("EUR:6.41"), DecimalNumber("4.69")))
@@ -250,7 +264,7 @@ class AmountTest {
     @Test
     fun conversionRevert() = dbSetup { db ->
         db.conn { conn ->
-            fun TalerAmount.apply(ratio: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount? {
+            fun TalerAmount.apply(ratio: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount {
                 val stmt = conn.prepareStatement("SELECT amount.val, amount.frac FROM conversion_apply_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode) as amount")
                 stmt.setLong(1, this.value)
                 stmt.setInt(2, this.frac)
@@ -259,16 +273,16 @@ class AmountTest {
                 stmt.setLong(5, tiny.value)
                 stmt.setInt(6, tiny.frac)
                 stmt.setString(7, roundingMode)
-                return stmt.oneOrNull {
+                return stmt.one {
                     TalerAmount(
                         it.getLong(1),
                         it.getInt(2),
                         currency
                     )
-                }!!
+                }
             }
 
-            fun TalerAmount.revert(ratio: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount? {
+            fun TalerAmount.revert(ratio: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount {
                 val stmt = conn.prepareStatement("SELECT amount.val, amount.frac FROM conversion_revert_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode) as amount")
                 stmt.setLong(1, this.value)
                 stmt.setInt(2, this.frac)
@@ -277,13 +291,13 @@ class AmountTest {
                 stmt.setLong(5, tiny.value)
                 stmt.setInt(6, tiny.frac)
                 stmt.setString(7, roundingMode)
-                return stmt.oneOrNull {
+                return stmt.one {
                     TalerAmount(
                         it.getLong(1),
                         it.getInt(2),
                         currency
                     )
-                }!!
+                }
             }
     
             assertEquals(TalerAmount("EUR:6.41"), TalerAmount("EUR:30.0629").revert(DecimalNumber("4.69")))
@@ -303,11 +317,11 @@ class AmountTest {
                             val ratio = DecimalNumber(ratio)
                             val base = TalerAmount("EUR:$amount")
                             // Apply ratio
-                            val rounded = base.apply(ratio, tiny, mode)!!
+                            val rounded = base.apply(ratio, tiny, mode)
                             // Revert ratio  
-                            val revert = rounded.revert(ratio, tiny, mode)!!
+                            val revert = rounded.revert(ratio, tiny, mode)
                             // Check applying ratio again give the same result
-                            val check = revert.apply(ratio, tiny, mode)!!
+                            val check = revert.apply(ratio, tiny, mode)
                             assertEquals(rounded, check)
                         }
                     }
@@ -317,7 +331,7 @@ class AmountTest {
     }
 
     @Test
-    fun apiError() = bankSetup { _ -> 
+    fun apiError() = bankSetup { 
         val base = obj {
             "payto_uri" to "$exchangePayto?message=payout"
         }
