@@ -30,27 +30,42 @@ import kotlin.test.assertEquals
 class BankIntegrationApiTest {
     // GET /taler-integration/config
     @Test
-    fun config() = bankSetup { _ ->
+    fun config() = bankSetup {
         client.get("/taler-integration/config").assertOk()
     }
 
     // GET /taler-integration/withdrawal-operation/UUID
     @Test
-    fun get() = bankSetup { _ ->
-        val amount = TalerAmount("KUDOS:9.0")
+    fun get() = bankSetup {
         // Check OK
-        client.postA("/accounts/customer/withdrawals") {
-            json { "amount" to amount } 
-        }.assertOkJson<BankAccountCreateWithdrawalResponse> { resp ->
-            val uuid = resp.taler_withdraw_uri.split("/").last()
-            client.get("/taler-integration/withdrawal-operation/$uuid")
-                .assertOkJson<BankWithdrawalOperationStatus> {
-                assert(!it.selection_done)
-                assert(!it.aborted)
-                assert(!it.transfer_done)
-                assertEquals(amount, it.amount)
-                assertEquals(listOf("iban"), it.wire_types)
-                assertEquals(amount, it.amount)
+        for (valid in listOf(
+            Pair(null, null),
+            Pair("KUDOS:1.0", null),
+            Pair(null, "KUDOS:2.0") ,
+            Pair("KUDOS:3.0", "KUDOS:4.0")
+        )) {
+            val amount = valid.first?.run(::TalerAmount)
+            val suggested = valid.second?.run(::TalerAmount)
+            client.postA("/accounts/merchant/withdrawals") {
+                json { 
+                    "amount" to amount
+                    "suggested_amount" to suggested
+                }
+            }.assertOkJson<BankAccountCreateWithdrawalResponse> {
+                val uuid = it.taler_withdraw_uri.split("/").last()
+                client.get("/taler-integration/withdrawal-operation/$uuid")
+                    .assertOkJson<BankWithdrawalOperationStatus> {
+                    assert(!it.selection_done)
+                    assert(!it.aborted)
+                    assert(!it.transfer_done)
+                    assertEquals(it.card_fees, TalerAmount.zero("KUDOS"))
+                    assertEquals(it.min_amount, TalerAmount.zero("KUDOS"))
+                    assertEquals(it.max_amount, TalerAmount("KUDOS:10"))
+                    assertEquals(amount, it.amount)
+                    assertEquals(suggested, it.suggested_amount)
+                    assertEquals(listOf("iban"), it.wire_types)
+                    assertEquals("KUDOS", it.currency)
+                }
             }
         }
 
@@ -68,7 +83,7 @@ class BankIntegrationApiTest {
 
     // POST /taler-integration/withdrawal-operation/UUID
     @Test
-    fun select() = bankSetup { _ ->
+    fun select() = bankSetup {
         val reserve_pub = EddsaPublicKey.rand()
         val req = obj {
             "reserve_pub" to reserve_pub
@@ -88,7 +103,7 @@ class BankIntegrationApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
 
             // Check OK
             client.post("/taler-integration/withdrawal-operation/$uuid") {
@@ -115,12 +130,20 @@ class BankIntegrationApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
 
             // Check reserve_pub_reuse
             client.post("/taler-integration/withdrawal-operation/$uuid") {
                 json(req)
             }.assertConflict(TalerErrorCode.BANK_DUPLICATE_RESERVE_PUB_SUBJECT)
+
+            // Check amount differs
+            client.post("/taler-integration/withdrawal-operation/$uuid") {
+                json(req) {
+                    "amount" to "KUDOS:2"
+                }
+            }.assertConflict(TalerErrorCode.BANK_AMOUNT_DIFFERS)
+
             // Check unknown account
             client.post("/taler-integration/withdrawal-operation/$uuid") {
                 json {
@@ -128,6 +151,7 @@ class BankIntegrationApiTest {
                     "selected_exchange" to unknownPayto
                 }
             }.assertConflict(TalerErrorCode.BANK_UNKNOWN_ACCOUNT)
+
             // Check account not exchange
             client.post("/taler-integration/withdrawal-operation/$uuid") {
                 json {
@@ -135,17 +159,87 @@ class BankIntegrationApiTest {
                     "selected_exchange" to merchantPayto
                 }
             }.assertConflict(TalerErrorCode.BANK_ACCOUNT_IS_NOT_EXCHANGE)
+
+            client.post("/taler-integration/withdrawal-operation/$uuid") {
+                json {
+                    "reserve_pub" to EddsaPublicKey.rand()
+                    "selected_exchange" to exchangePayto.canonical
+                    "amount" to "KUDOS:1"
+                }
+            }.assertOkJson<BankWithdrawalOperationPostResponse>()
         }
+
+        client.postA("/accounts/merchant/withdrawals") {
+            json {}
+        }.assertOkJson<BankAccountCreateWithdrawalResponse> {
+            val uuid = it.withdrawal_id
+
+            // Check missing amount
+            client.post("/taler-integration/withdrawal-operation/$uuid") {
+                json {
+                    "reserve_pub" to EddsaPublicKey.rand()
+                    "selected_exchange" to exchangePayto.canonical
+                }
+            }.assertConflict(TalerErrorCode.BANK_AMOUNT_REQUIRED)
+
+            // Check insufficient fund
+            client.post("/taler-integration/withdrawal-operation/$uuid") {
+                json {
+                    "reserve_pub" to EddsaPublicKey.rand()
+                    "selected_exchange" to exchangePayto.canonical
+                    "amount" to "KUDOS:11"
+                }
+            }.assertConflict(TalerErrorCode.BANK_UNALLOWED_DEBIT)
+
+            client.post("/taler-integration/withdrawal-operation/$uuid") {
+                json {
+                    "reserve_pub" to EddsaPublicKey.rand()
+                    "selected_exchange" to exchangePayto.canonical
+                    "amount" to "KUDOS:1.1"
+                }
+            }.assertOkJson<BankWithdrawalOperationPostResponse>()
+            client.get("/taler-integration/withdrawal-operation/$uuid")
+                .assertOkJson<BankWithdrawalOperationStatus> {
+                assertEquals(TalerAmount("KUDOS:1.1"), it.amount)
+                assertEquals(TalerAmount("KUDOS:10"), it.max_amount)
+            }
+        }
+    }
+
+    @Test
+    fun selectWithFee() = bankSetup(conf = "test_with_fees.conf") {
+        val uuid = client.postA("/accounts/merchant/withdrawals") {
+            json {}
+        }.assertOkJson<BankAccountCreateWithdrawalResponse>().withdrawal_id
+        // Check insufficient fund
+        for (amount in listOf("KUDOS:11", "KUDOS:10", "KUDOS:0", "KUDOS:150")) {
+            client.post("/taler-integration/withdrawal-operation/$uuid") {
+                json {
+                    "reserve_pub" to EddsaPublicKey.rand()
+                    "selected_exchange" to exchangePayto.canonical
+                    "amount" to amount
+                }
+            }.assertConflict(TalerErrorCode.BANK_UNALLOWED_DEBIT)
+        }
+
+        // Check OK
+        client.post("/taler-integration/withdrawal-operation/$uuid") {
+            json {
+                "reserve_pub" to EddsaPublicKey.rand()
+                "selected_exchange" to exchangePayto.canonical
+                "amount" to "KUDOS:9"
+            }
+        }.assertOk()
     }
 
     // POST /taler-integration/withdrawal-operation/UUID/abort
     @Test
-    fun abort() = bankSetup { _ ->
+    fun abort() = bankSetup {
         // Check abort created
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
 
             // Check OK
             client.postA("/taler-integration/withdrawal-operation/$uuid/abort").assertNoContent()
@@ -157,7 +251,7 @@ class BankIntegrationApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
             withdrawalSelect(uuid)
 
             // Check OK
@@ -170,7 +264,7 @@ class BankIntegrationApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
             withdrawalSelect(uuid)
             client.postA("/accounts/merchant/withdrawals/$uuid/confirm").assertNoContent()
 
