@@ -26,13 +26,13 @@ import kotlin.test.assertEquals
 class ConversionApiTest {
     // GET /conversion-info/config
     @Test
-    fun config() = bankSetup { _ ->
+    fun config() = bankSetup {
         client.get("/conversion-info/config").assertOk()
     }
     
     // GET /conversion-info/cashout-rate
     @Test
-    fun cashoutRate() = bankSetup { _ ->
+    fun cashoutRate() = bankSetup {
         // Check conversion to
         client.get("/conversion-info/cashout-rate?amount_debit=KUDOS:1").assertOkJson<ConversionResponse> {
             assertEquals(TalerAmount("KUDOS:1"), it.amount_debit)
@@ -67,7 +67,7 @@ class ConversionApiTest {
 
     // GET /conversion-info/cashin-rate
     @Test
-    fun cashinRate() = bankSetup { _ ->
+    fun cashinRate() = bankSetup {
         for ((amount, converted) in listOf(
             Pair(0.75, 0.58), Pair(0.32, 0.24), Pair(0.66, 0.51)
         )) {
@@ -117,7 +117,7 @@ class ConversionApiTest {
     }
 
     @Test
-    fun notImplemented() = bankSetup("test_no_conversion.conf") { _ ->
+    fun notImplemented() = bankSetup("test_no_conversion.conf") {
         client.get("/conversion-info/cashin-rate")
             .assertNotImplemented()
         client.get("/conversion-info/cashout-rate")
