@@ -24,8 +24,10 @@ import io.ktor.server.testing.*
 import kotlinx.serialization.json.JsonElement
 import org.junit.Test
 import tech.libeufin.bank.*
-import tech.libeufin.bank.auth.*
+import tech.libeufin.bank.auth.TOKEN_PREFIX
 import tech.libeufin.common.*
+import tech.libeufin.common.crypto.CryptoUtil
+import tech.libeufin.common.db.one
 import java.time.Duration
 import java.time.Instant
 import java.util.*
@@ -33,16 +35,52 @@ import kotlin.test.assertEquals
 import kotlin.test.assertNotNull
 import kotlin.test.assertNull
 
+class CoreBankSecurityTest {
+    @Test
+    fun passwordUpdate() = bankSetup { db ->
+        suspend fun currentHash(): String {
+            return db.serializable(
+                "SELECT password_hash FROM customers WHERE login='customer'"
+            ) {
+                one {
+                    it.getString(1)
+                }
+            }
+        }
+
+        // Set outdated hash
+        val password = "customer-password"
+        val pwh = CryptoUtil.hashStringSHA256(password).encodeBase64()
+        val hash = "sha256\$$pwh"
+        db.serializable(
+            "UPDATE customers SET password_hash=? WHERE login='customer'"
+        ) {
+            setString(1, hash)
+            executeUpdate()
+        }
+        assertEquals(hash, currentHash())
+
+        // Check hash is updated
+        client.getA("/accounts/customer").assertOk()
+        val newHash = currentHash()
+        assert(hash != newHash)
+
+        // Check hash stay the same
+        client.getA("/accounts/customer").assertOk()
+        assertEquals(newHash, currentHash())
+    }
+}
+
 class CoreBankConfigTest {
     // GET /config
     @Test
-    fun config() = bankSetup { _ -> 
+    fun config() = bankSetup { 
         client.get("/config").assertOk()
     }
 
     // GET /monitor
     @Test
-    fun monitor() = bankSetup { _ -> 
+    fun monitor() = bankSetup { 
         authRoutine(HttpMethod.Get, "/monitor", requireAdmin = true)
         // Check OK
         client.get("/monitor?timeframe=day&which=25") {
@@ -65,7 +103,7 @@ class CoreBankTokenApiTest {
             json { "scope" to "readonly" }
         }.assertOkJson<TokenSuccessResponse> {
             // Checking that the token lifetime defaulted to 24 hours.
-            val token = db.token.get(Base32Crockford.decode(it.access_token.removePrefix(TOKEN_PREFIX)))
+            val token = db.token.access(Base32Crockford.decode(it.access_token.removePrefix(TOKEN_PREFIX)), Instant.now())
             val lifeTime = Duration.between(token!!.creationTime, token.expirationTime)
             assertEquals(Duration.ofDays(1), lifeTime)
         }
@@ -75,26 +113,68 @@ class CoreBankTokenApiTest {
             json { "scope" to "readonly" }
         }.assertOkJson<TokenSuccessResponse> {
             // Checking that the token lifetime defaulted to 24 hours.
-            val token = db.token.get(Base32Crockford.decode(it.access_token.removePrefix(TOKEN_PREFIX)))
+            val token = db.token.access(Base32Crockford.decode(it.access_token.removePrefix(TOKEN_PREFIX)), Instant.now())
             val lifeTime = Duration.between(token!!.creationTime, token.expirationTime)
             assertEquals(Duration.ofDays(1), lifeTime)
         }
 
-        // Check refresh
+        // Check valid refresh scope
+        for ((fromScope, toScope) in listOf(
+            "readwrite" to "readwrite",
+            "readonly" to "readonly",
+            "revenue" to "revenue",
+            "readwrite" to "readonly",
+            "readwrite" to "revenue",
+            "readonly" to "revenue",
+        )) {
+            client.postA("/accounts/merchant/token") {
+                json { 
+                    "scope" to fromScope
+                    "refreshable" to true
+                }
+            }.assertOkJson<TokenSuccessResponse> {
+                val token = it.access_token
+                client.post("/accounts/merchant/token") {
+                    headers[HttpHeaders.Authorization] = "Bearer $token"
+                    json { "scope" to toScope }
+                }.assertOk()
+            }
+        }
+
+        // Check invalid refresh scope
+        for ((fromScope, toScope) in listOf(
+            "readonly" to "readwrite",
+            "revenue" to "readonly",
+            "revenue" to "readwrite"
+        )) {
+            client.postA("/accounts/merchant/token") {
+                json { 
+                    "scope" to fromScope
+                    "refreshable" to true
+                }
+            }.assertOkJson<TokenSuccessResponse> {
+                val token = it.access_token
+                client.post("/accounts/merchant/token") {
+                    headers[HttpHeaders.Authorization] = "Bearer $token"
+                    json { "scope" to toScope }
+                }.assertForbidden(TalerErrorCode.GENERIC_TOKEN_PERMISSION_INSUFFICIENT)
+            }
+        }
+
+        // Check no refreshable
         client.postA("/accounts/merchant/token") {
             json { 
                 "scope" to "readonly"
-                "refreshable" to true
             }
         }.assertOkJson<TokenSuccessResponse> {
             val token = it.access_token
             client.post("/accounts/merchant/token") {
-                headers["Authorization"] = "Bearer $token"
+                headers[HttpHeaders.Authorization] = "Bearer $token"
                 json { "scope" to "readonly" }
-            }.assertOk()
+            }.assertUnauthorized()
         }
         
-        // Check'forever' case.
+        // Check 'forever' case.
         client.postA("/accounts/merchant/token") {
             json { 
                 "scope" to "readonly"
@@ -136,29 +216,60 @@ class CoreBankTokenApiTest {
 
     // DELETE /accounts/USERNAME/token
     @Test
-    fun delete() = bankSetup { _ -> 
+    fun delete() = bankSetup { 
         // TODO test restricted
         val token = client.postA("/accounts/merchant/token") {
             json { "scope" to "readonly" }
         }.assertOkJson<TokenSuccessResponse>().access_token
         // Check OK
         client.delete("/accounts/merchant/token") {
-            headers["Authorization"] = "Bearer $token"
+            headers[HttpHeaders.Authorization] = "Bearer $token"
         }.assertNoContent()
         // Check token no longer work
         client.delete("/accounts/merchant/token") {
-            headers["Authorization"] = "Bearer $token"
+            headers[HttpHeaders.Authorization] = "Bearer $token"
         }.assertUnauthorized()
 
         // Checking merchant can still be served by basic auth, after token deletion.
         client.getA("/accounts/merchant").assertOk()
     }
+
+    // GET /accounts/USERNAME/token
+    @Test
+    fun get() = bankSetup {
+        // Check OK
+        for (account in listOf("merchant", "customer")) {
+            client.getA("/accounts/$account/tokens").assertNoContent()
+        }
+        client.postA("/accounts/merchant/token") {
+            json { "scope" to "readonly" }
+        }.assertOk()
+        client.postA("/accounts/merchant/token") {
+            json { "scope" to "readwrite" }
+        }.assertOk()
+        client.postA("/accounts/customer/token") {
+            json {
+                "scope" to "revenue"
+                "description" to "description"
+            }
+        }.assertOk()
+        client.getA("/accounts/merchant/tokens").assertOkJson<TokenInfos> {
+            assertEquals(2, it.tokens.size)
+            for (token in it.tokens) {
+                assertNull(token.description)
+            }
+        }
+        client.getA("/accounts/customer/tokens").assertOkJson<TokenInfos> {
+            assertEquals(1, it.tokens.size)
+            assertEquals("description", it.tokens[0].description)
+        }
+    }
 }
 
 class CoreBankAccountsApiTest {
     // Testing the account creation and its idempotency
     @Test
-    fun create() = bankSetup { _ -> 
+    fun create() = bankSetup { 
         // Check generated payto
         obj {
             "username" to "john"
@@ -376,7 +487,7 @@ class CoreBankAccountsApiTest {
 
     // Test account created with bonus
     @Test
-    fun createBonus() = bankSetup(conf = "test_bonus.conf") { _ ->
+    fun createBonus() = bankSetup(conf = "test_bonus.conf") {
         val req = obj {
             "username" to "foo"
             "password" to "xyz"
@@ -411,7 +522,7 @@ class CoreBankAccountsApiTest {
 
     // Test admin-only account creation
     @Test
-    fun createRestricted() = bankSetup(conf = "test_restrict.conf") { _ -> 
+    fun createRestricted() = bankSetup(conf = "test_restrict.conf") { 
         authRoutine(HttpMethod.Post, "/accounts", requireAdmin = true)
         client.post("/accounts") {
             pwAuth("admin")
@@ -425,7 +536,7 @@ class CoreBankAccountsApiTest {
 
     // Test admin-only account creation
     @Test
-    fun createTanErr() = bankSetup(conf = "test_tan_err.conf") { _ -> 
+    fun createTanErr() = bankSetup(conf = "test_tan_err.conf") { 
         client.post("/accounts") {
             pwAuth("admin")
             json {
@@ -516,7 +627,7 @@ class CoreBankAccountsApiTest {
         
         // Check account can no longer login
         client.delete("/accounts/customer/token") {
-            headers["Authorization"] = "Bearer $token"
+            headers[HttpHeaders.Authorization] = "Bearer $token"
         }.assertUnauthorized()
         client.getA("/accounts/customer/transactions/$tx_id").assertUnauthorized()
         client.getA("/accounts/customer/cashouts/$cashout_id").assertUnauthorized()
@@ -546,7 +657,7 @@ class CoreBankAccountsApiTest {
 
     // Test admin-only account deletion
     @Test
-    fun deleteRestricted() = bankSetup(conf = "test_restrict.conf") { _ -> 
+    fun deleteRestricted() = bankSetup(conf = "test_restrict.conf") { 
         authRoutine(HttpMethod.Post, "/accounts", requireAdmin = true)
         // Exchange is still restricted
         client.delete("/accounts/exchange") {
@@ -556,7 +667,7 @@ class CoreBankAccountsApiTest {
 
     // Test delete exchange account
     @Test
-    fun deleteNoConversion() = bankSetup(conf = "test_no_conversion.conf") { _ -> 
+    fun deleteNoConversion() = bankSetup(conf = "test_no_conversion.conf") { 
         // Exchange is no longer restricted
         client.deleteA("/accounts/exchange").assertNoContent()
     }
@@ -582,7 +693,7 @@ class CoreBankAccountsApiTest {
 
     // PATCH /accounts/USERNAME
     @Test
-    fun reconfig() = bankSetup { _ -> 
+    fun reconfig() = bankSetup { 
         authRoutine(HttpMethod.Patch, "/accounts/merchant", allowAdmin = true)
 
         // Check tan info
@@ -701,7 +812,7 @@ class CoreBankAccountsApiTest {
 
     // Test admin-only account patch
     @Test
-    fun patchRestricted() = bankSetup(conf = "test_restrict.conf") { _ -> 
+    fun patchRestricted() = bankSetup(conf = "test_restrict.conf") { 
         // Check restricted
         checkAdminOnly(
             obj { "name" to "Another Foo" },
@@ -725,7 +836,7 @@ class CoreBankAccountsApiTest {
 
     // Test TAN check account patch
     @Test
-    fun patchTanErr() = bankSetup(conf = "test_tan_err.conf") { _ -> 
+    fun patchTanErr() = bankSetup(conf = "test_tan_err.conf") { 
         // Check unsupported TAN channel
         client.patchA("/accounts/customer") {
             json {
@@ -736,7 +847,7 @@ class CoreBankAccountsApiTest {
 
     // PATCH /accounts/USERNAME/auth
     @Test
-    fun passwordChange() = bankSetup { _ -> 
+    fun passwordChange() = bankSetup { 
         authRoutine(HttpMethod.Patch, "/accounts/merchant/auth", allowAdmin = true)
 
         // Changing the password.
@@ -873,7 +984,7 @@ class CoreBankAccountsApiTest {
 
     // GET /accounts/USERNAME
     @Test
-    fun get() = bankSetup { _ -> 
+    fun get() = bankSetup { 
         authRoutine(HttpMethod.Get, "/accounts/merchant", allowAdmin = true)
         // Check ok
         client.getA("/accounts/merchant").assertOkJson<AccountData> {
@@ -885,7 +996,7 @@ class CoreBankAccountsApiTest {
 class CoreBankTransactionsApiTest {
     // GET /transactions
     @Test
-    fun history() = bankSetup { _ -> 
+    fun history() = bankSetup { 
         authRoutine(HttpMethod.Get, "/accounts/merchant/transactions", allowAdmin = true)
         historyRoutine<BankAccountTransactionsResponse>(
             url = "/accounts/customer/transactions",
@@ -915,7 +1026,7 @@ class CoreBankTransactionsApiTest {
                 },
                 {
                     // Ignore transactions of other accounts
-                    tx("exchange", "KUDOS:0.1", "merchant",)
+                    tx("exchange", "KUDOS:0.1", "merchant")
                 }
             )
         )
@@ -923,7 +1034,7 @@ class CoreBankTransactionsApiTest {
 
     // GET /transactions/T_ID
     @Test
-    fun testById() = bankSetup { _ -> 
+    fun testById() = bankSetup { 
         authRoutine(HttpMethod.Get, "/accounts/merchant/transactions/42", allowAdmin = true)
 
         // Create transaction
@@ -1134,19 +1245,71 @@ class CoreBankTransactionsApiTest {
             }
         }.assertConflict(TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED)
     }
+
+    @Test
+    fun createWithFee() = bankSetup(conf = "test_with_fees.conf") {
+        // Init state
+        assertBalance("merchant", "+KUDOS:0")
+        assertBalance("customer", "+KUDOS:0")
+        assertBalance("admin", "+KUDOS:0")
+
+        // Check fee are sent to admin
+        tx("merchant", "KUDOS:3", "customer")
+        assertBalance("merchant", "-KUDOS:3.1")
+        assertBalance("customer", "+KUDOS:3")
+        assertBalance("admin", "+KUDOS:0.1")
+
+        // Check amount with fee and min & max are checked
+        for (amount in listOf("KUDOS:7", "KUDOS:6.9", "KUDOS:0", "KUDOS:150")) {
+            client.postA("/accounts/merchant/transactions") {
+                json {
+                    "payto_uri" to "$customerPayto?message=payout2&amount=$amount"
+                }
+            }.assertConflict(TalerErrorCode.BANK_UNALLOWED_DEBIT)
+        }
+        // Check empty account
+        tx("merchant", "KUDOS:6.8", "customer")
+        assertBalance("merchant", "-KUDOS:10")
+        assertBalance("customer", "+KUDOS:9.8")
+        assertBalance("admin", "+KUDOS:0.2")
+
+        // Admin check no fee
+        tx("admin", "KUDOS:0.35", "merchant")
+        assertBalance("merchant", "-KUDOS:9.65")
+        assertBalance("admin", "-KUDOS:0.15")
+
+        // Admin recover from debt
+        tx("customer", "KUDOS:1", "merchant")
+        assertBalance("admin", "-KUDOS:0.05")
+        tx("customer", "KUDOS:1", "merchant")
+        assertBalance("merchant", "-KUDOS:7.65")
+        assertBalance("customer", "+KUDOS:7.6")
+        assertBalance("admin", "+KUDOS:0.05")
+    }
 }
 
 class CoreBankWithdrawalApiTest {
     // POST /accounts/USERNAME/withdrawals
     @Test
-    fun create() = bankSetup { _ ->
+    fun create() = bankSetup {
         authRoutine(HttpMethod.Post, "/accounts/merchant/withdrawals")
         
         // Check OK
-        client.postA("/accounts/merchant/withdrawals") {
-            json { "amount" to "KUDOS:9.0" } 
-        }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            assertEquals("taler+http://withdraw/localhost:80/taler-integration/${it.withdrawal_id}", it.taler_withdraw_uri)
+        for (valid in listOf(
+            obj {}, 
+            obj { "amount" to "KUDOS:1.0" },
+            obj { "suggested_amount" to "KUDOS:2.0" }, 
+            obj {
+                "amount" to "KUDOS:3.0"
+                "suggested_amount" to "KUDOS:4.0"
+            }
+        )) {
+            // Check OK
+            client.postA("/accounts/merchant/withdrawals") {
+                json(valid)
+            }.assertOkJson<BankAccountCreateWithdrawalResponse> {
+                assertEquals("taler+http://withdraw/localhost:80/taler-integration/${it.withdrawal_id}", it.taler_withdraw_uri)
+            }
         }
 
         // Check exchange account
@@ -1158,18 +1321,61 @@ class CoreBankWithdrawalApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:90" } 
         }.assertConflict(TalerErrorCode.BANK_UNALLOWED_DEBIT)
+        client.postA("/accounts/merchant/withdrawals") {
+            json { "suggested_amount" to "KUDOS:90" } 
+        }.assertConflict(TalerErrorCode.BANK_UNALLOWED_DEBIT)
+
+        // Check wrong currency
+        client.postA("/accounts/merchant/withdrawals") {
+            json { "amount" to "EUR:90" } 
+        }.assertBadRequest(TalerErrorCode.GENERIC_CURRENCY_MISMATCH)
+        client.postA("/accounts/merchant/withdrawals") {
+            json { "suggested_amount" to "EUR:90" } 
+        }.assertBadRequest(TalerErrorCode.GENERIC_CURRENCY_MISMATCH)
+    }
+    
+    @Test
+    fun createWithFee() = bankSetup(conf = "test_with_fees.conf") {
+        // Check insufficient fund
+        for (amount in listOf("KUDOS:11", "KUDOS:10", "KUDOS:0", "KUDOS:150")) {
+            for (name in listOf("amount", "suggested_amount")) {
+                client.postA("/accounts/merchant/withdrawals") {
+                    json { name to amount } 
+                }.assertConflict(TalerErrorCode.BANK_UNALLOWED_DEBIT)
+            }
+        }
+
+        // Check OK
+        for (name in listOf("amount", "suggested_amount")) {
+            client.postA("/accounts/merchant/withdrawals") {
+                json { name to "KUDOS:9.9" } 
+            }.assertOk()
+        }
     }
 
     // GET /withdrawals/withdrawal_id
     @Test
-    fun get() = bankSetup { _ ->
-        val amount = TalerAmount("KUDOS:9.0")
+    fun get() = bankSetup {
         // Check OK
-        client.postA("/accounts/merchant/withdrawals") {
-            json { "amount" to amount}
-        }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            client.get("/withdrawals/${it.withdrawal_id}").assertOkJson<WithdrawalPublicInfo> {
-                assertEquals(amount, it.amount)
+        for (valid in listOf(
+            Pair(null, null),
+            Pair("KUDOS:1.0", null),
+            Pair(null, "KUDOS:2.0") ,
+            Pair("KUDOS:3.0", "KUDOS:4.0")
+        )) {
+            val amount = valid.first?.run(::TalerAmount)
+            val suggested = valid.second?.run(::TalerAmount)
+            client.postA("/accounts/merchant/withdrawals") {
+                json { 
+                    "amount" to amount
+                    "suggested_amount" to suggested
+                }
+            }.assertOkJson<BankAccountCreateWithdrawalResponse> {
+                client.get("/withdrawals/${it.withdrawal_id}")
+                    .assertOkJson<WithdrawalPublicInfo> {
+                    assertEquals(amount, it.amount)
+                    assertEquals(suggested, it.suggested_amount)
+                }
             }
         }
 
@@ -1186,7 +1392,7 @@ class CoreBankWithdrawalApiTest {
 
     // POST /accounts/USERNAME/withdrawals/withdrawal_id/confirm
     @Test
-    fun confirm() = bankSetup { _ -> 
+    fun confirm() = bankSetup { 
         authRoutine(HttpMethod.Post, "/accounts/merchant/withdrawals/42/confirm")
         // Check confirm created
         client.postA("/accounts/merchant/withdrawals") {
@@ -1276,12 +1482,40 @@ class CoreBankWithdrawalApiTest {
             }.assertNoContent()
         }
     }
+
+    @Test
+    fun confirmWithFee() = bankSetup(conf = "test_with_fees.conf") { db ->
+        suspend fun run(amount: TalerAmount): HttpResponse {
+            val uuid = UUID.randomUUID()
+            // Create a selected withdrawal directly in the database to bypass checks
+            db.serializable("""
+                INSERT INTO taler_withdrawal_operations(withdrawal_uuid,amount,selected_exchange_payto,selection_done,wallet_bank_account,creation_date)
+                VALUES (?, (?, ?)::taler_amount, ?, true, 3, 0)
+            """) {
+                setObject(1, uuid)
+                setLong(2, amount.value)
+                setInt(3, amount.frac)
+                setString(4, exchangePayto.canonical)
+                execute()
+            }
+
+            return client.postA("/accounts/customer/withdrawals/$uuid/confirm")
+        }
+
+        // Check insufficient fund
+        for (amount in listOf("KUDOS:11", "KUDOS:10", "KUDOS:0", "KUDOS:150")) {
+            run(TalerAmount(amount)).assertConflict(TalerErrorCode.BANK_UNALLOWED_DEBIT)
+        }
+
+        // Check OK
+        run(TalerAmount("KUDOS:9.9"))
+    }
 }
 
 class CoreBankCashoutApiTest {
     // POST /accounts/{USERNAME}/cashouts
     @Test
-    fun create() = bankSetup { _ ->
+    fun create() = bankSetup {
         authRoutine(HttpMethod.Post, "/accounts/merchant/cashouts")
 
         val req = obj {
@@ -1396,7 +1630,7 @@ class CoreBankCashoutApiTest {
 
     // GET /accounts/{USERNAME}/cashouts/{CASHOUT_ID}
     @Test
-    fun get() = bankSetup { _ ->
+    fun get() = bankSetup {
         authRoutine(HttpMethod.Get, "/accounts/merchant/cashouts/42", allowAdmin = true)
         fillCashoutInfo("customer")
 
@@ -1444,31 +1678,31 @@ class CoreBankCashoutApiTest {
 
     // GET /accounts/{USERNAME}/cashouts
     @Test
-    fun history() = bankSetup { _ ->
+    fun history() = bankSetup {
         authRoutine(HttpMethod.Get, "/accounts/merchant/cashouts", allowAdmin = true)
         historyRoutine<Cashouts>(
             url = "/accounts/customer/cashouts",
             ids = { it.cashouts.map { it.cashout_id } },
-            registered = listOf({ cashout("KUDOS:0.1") }),
+            registered = listOf { cashout("KUDOS:0.1") },
             polling = false
         )
     }
 
     // GET /cashouts
     @Test
-    fun globalHistory() = bankSetup { _ ->
+    fun globalHistory() = bankSetup {
         authRoutine(HttpMethod.Get, "/cashouts", requireAdmin = true)
         historyRoutine<GlobalCashouts>(
             url = "/cashouts",
             ids = { it.cashouts.map { it.cashout_id } },
-            registered = listOf({ cashout("KUDOS:0.1") }),
+            registered = listOf { cashout("KUDOS:0.1") },
             polling = false,
             auth = "admin"
         )
     }
 
     @Test
-    fun notImplemented() = bankSetup("test_no_conversion.conf") { _ ->
+    fun notImplemented() = bankSetup("test_no_conversion.conf") {
         client.get("/accounts/customer/cashouts")
             .assertNotImplemented()
     }
@@ -1477,7 +1711,7 @@ class CoreBankCashoutApiTest {
 class CoreBankTanApiTest {
     // POST /accounts/{USERNAME}/challenge/{challenge_id}
     @Test
-    fun send() = bankSetup { _ ->
+    fun send() = bankSetup {
         authRoutine(HttpMethod.Post, "/accounts/merchant/challenge/42")
 
         suspend fun HttpResponse.expectChallenge(channel: TanChannel, info: String): HttpResponse {
@@ -1602,7 +1836,7 @@ class CoreBankTanApiTest {
 
     // POST /accounts/{USERNAME}/challenge/{challenge_id}
     @Test
-    fun sendTanErr() = bankSetup("test_tan_err.conf") { _ ->
+    fun sendTanErr() = bankSetup("test_tan_err.conf") {
         // Check fail
         client.patch("/accounts/merchant") {
             pwAuth("admin")
@@ -1621,7 +1855,7 @@ class CoreBankTanApiTest {
 
     // POST /accounts/{USERNAME}/challenge/{challenge_id}/confirm
     @Test
-    fun confirm() = bankSetup { _ ->
+    fun confirm() = bankSetup {
         authRoutine(HttpMethod.Post, "/accounts/merchant/challenge/42/confirm")
 
         fillTanInfo("merchant")
