@@ -19,12 +19,13 @@
 
 import io.ktor.http.*
 import org.junit.Test
-import tech.libeufin.common.*
+import tech.libeufin.common.RevenueIncomingHistory
+import tech.libeufin.common.assertOk
 
 class RevenueApiTest {
     // GET /accounts/{USERNAME}/taler-revenue/config
     @Test
-    fun config() = bankSetup { _ ->
+    fun config() = bankSetup {
         authRoutine(HttpMethod.Get, "/accounts/merchant/taler-revenue/config")
 
         client.getA("/accounts/merchant/taler-revenue/config").assertOk()
