@@ -23,8 +23,11 @@ import tech.libeufin.bank.MonitorParams
 import tech.libeufin.bank.MonitorResponse
 import tech.libeufin.bank.MonitorWithConversion
 import tech.libeufin.bank.Timeframe
+import tech.libeufin.common.ShortHashCode
+import tech.libeufin.common.TalerAmount
+import tech.libeufin.common.assertOkJson
 import tech.libeufin.common.db.executeQueryCheck
-import tech.libeufin.common.*
+import tech.libeufin.common.micros
 import java.time.Instant
 import java.time.LocalDateTime
 import java.time.ZoneOffset
@@ -61,7 +64,6 @@ class StatsTest {
             Timeframe.entries.forEach { timeframe -> 
                 client.get("/monitor?timestamp=${timeframe.name}") { pwAuth("admin") }.assertOkJson<MonitorResponse> {
                     val resp = it as MonitorWithConversion
-                    println("$resp")
                     assertEquals(count, dbCount(resp))
                     assertEquals(TalerAmount(regionalAmount), regionalVolume(resp))
                     fiatVolume?.run { assertEquals(TalerAmount(fiatAmount!!), this(resp)) }
@@ -123,11 +125,11 @@ class StatsTest {
     @Test
     fun timeframe() = bankSetup { db ->
         db.conn { conn ->
-            suspend fun register(now: LocalDateTime, amount: TalerAmount) {
+            fun register(timestamp: LocalDateTime, amount: TalerAmount) {
                 val stmt = conn.prepareStatement(
                     "CALL stats_register_payment('taler_out', ?::timestamp, (?, ?)::taler_amount, null)"
                 )
-                stmt.setObject(1, now)
+                stmt.setObject(1, timestamp)
                 stmt.setLong(2, amount.value)
                 stmt.setInt(3, amount.frac)
                 stmt.executeUpdate()
@@ -144,18 +146,18 @@ class StatsTest {
             }
 
             suspend fun checkSimple(
-                now: LocalDateTime,
+                timestamp: LocalDateTime,
                 timeframe: Timeframe,
                 count: Long,
                 amount: TalerAmount
-            ) = check(MonitorParams(timeframe, now), count, amount)
+            ) = check(MonitorParams(timeframe, timestamp), count, amount)
             suspend fun checkWhich(
-                now: LocalDateTime,
+                timestamp: LocalDateTime,
                 timeframe: Timeframe,
                 which: Int,
                 count: Long,
                 amount: TalerAmount
-            ) = check(MonitorParams(timeframe, now, which), count, amount)
+            ) = check(MonitorParams(timeframe, timestamp, which), count, amount)
             suspend fun checkDate(
                 secs: Long,
                 timeframe: Timeframe,
