@@ -24,7 +24,7 @@ import tech.libeufin.common.*
 class WireGatewayApiTest {
     // GET /accounts/{USERNAME}/taler-wire-gateway/config
     @Test
-    fun config() = bankSetup { _ ->
+    fun config() = bankSetup {
         authRoutine(HttpMethod.Get, "/accounts/merchant/taler-wire-gateway/config")
 
         client.getA("/accounts/merchant/taler-wire-gateway/config").assertOk()
@@ -32,7 +32,7 @@ class WireGatewayApiTest {
 
     // POST /accounts/{USERNAME}/taler-wire-gateway/transfer
     @Test
-    fun transfer() = bankSetup { _ -> 
+    fun transfer() = bankSetup { 
         val valid_req = obj {
             "request_uid" to HashCode.rand()
             "amount" to "KUDOS:55"
@@ -190,7 +190,7 @@ class WireGatewayApiTest {
 
     // POST /accounts/{USERNAME}/taler-wire-gateway/admin/add-incoming
     @Test
-    fun addIncoming() = bankSetup { _ -> 
+    fun addIncoming() = bankSetup { 
         val valid_req = obj {
             "amount" to "KUDOS:44"
             "reserve_pub" to EddsaPublicKey.rand()
