@@ -1,6 +1,6 @@
 /*
  * This file is part of LibEuFin.
- * Copyright (C) 2023 Taler Systems S.A.
+ * Copyright (C) 2023-2024 Taler Systems S.A.
 
  * LibEuFin is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -18,7 +18,6 @@
  */
 
 import io.ktor.client.request.*
-import io.ktor.client.statement.*
 import io.ktor.http.*
 import io.ktor.server.testing.*
 import kotlinx.coroutines.coroutineScope
@@ -28,7 +27,8 @@ import kotlinx.serialization.json.JsonObject
 import tech.libeufin.bank.BankAccountCreateWithdrawalResponse
 import tech.libeufin.bank.WithdrawalStatus
 import tech.libeufin.common.*
-import tech.libeufin.common.test.*
+import tech.libeufin.common.test.abstractHistoryRoutine
+import tech.libeufin.common.test.assertTime
 import kotlin.test.assertEquals
 
 // Test endpoint is correctly authenticated 
@@ -50,7 +50,7 @@ suspend fun ApplicationTestBuilder.authRoutine(
     // Bad header
     client.request(path) {
         this.method = method
-        headers["Authorization"] = "WTF"
+        headers[HttpHeaders.Authorization] = "WTF"
     }.assertBadRequest(TalerErrorCode.GENERIC_HTTP_HEADERS_MALFORMED)
     
     // Unknown account
