@@ -23,7 +23,12 @@ const val MIN_VERSION: Int = 14
 const val SERIALIZATION_RETRY: Int = 10
 
 // Security
-const val MAX_BODY_LENGTH: Long = 4 * 1024 // 4kB
+const val MAX_BODY_LENGTH: Int = 4 * 1024 // 4kB
 
 // API version
-const val WIRE_GATEWAY_API_VERSION: String = "0:2:0"
\ No newline at end of file
+const val WIRE_GATEWAY_API_VERSION: String = "0:2:0"
+const val REVENUE_API_VERSION: String = "0:0:0"
+
+// HTTP headers
+const val X_CHALLENGE_ID: String = "X-Challenge-Id"
+const val X_FORWARD_PREFIX: String = "X-Forward-Prefix"
\ No newline at end of file
