@@ -20,6 +20,7 @@
 package tech.libeufin.common
 
 import io.ktor.http.*
+import io.ktor.server.plugins.*
 import kotlinx.serialization.KSerializer
 import kotlinx.serialization.Serializable
 import kotlinx.serialization.descriptors.PrimitiveKind
@@ -31,13 +32,10 @@ import kotlinx.serialization.json.JsonDecoder
 import kotlinx.serialization.json.JsonElement
 import kotlinx.serialization.json.jsonPrimitive
 import kotlinx.serialization.json.longOrNull
-import tech.libeufin.common.*
 import java.net.URI
 import java.net.URL
-import java.time.Duration
 import java.time.Instant
 import java.time.temporal.ChronoUnit
-import java.util.concurrent.TimeUnit
 
 sealed class CommonError(msg: String): Exception(msg) {
     class AmountFormat(msg: String): CommonError(msg)
@@ -67,7 +65,6 @@ data class TalerProtocolTimestamp(
             } else {
                 encoder.encodeLong(value.epochSecond)
             }
-            
         }
     
         override fun deserialize(decoder: Decoder): Instant {
@@ -92,12 +89,8 @@ data class TalerProtocolTimestamp(
 
 
 @Serializable(with = ExchangeUrl.Serializer::class)
-class ExchangeUrl {
-    val url: String
-
-    constructor(raw: String) {
-        url = URL(raw).toString()
-    }
+class ExchangeUrl(raw: String) {
+    val url: String = URL(raw).toString()
 
     override fun toString(): String = url
 
@@ -115,6 +108,69 @@ class ExchangeUrl {
     }
 }
 
+@Serializable(with = DecimalNumber.Serializer::class)
+class DecimalNumber {
+    val value: Long
+    val frac: Int
+
+    constructor(value: Long, frac: Int) {
+        this.value = value
+        this.frac = frac
+    }
+    constructor(encoded: String) {
+        val match = PATTERN.matchEntire(encoded) ?: throw badRequest("Invalid decimal number format")
+        val (value, frac) = match.destructured
+        this.value = value.toLongOrNull() ?: 
+            throw badRequest("Invalid value")
+        if (this.value > TalerAmount.MAX_VALUE) 
+            throw badRequest("Value specified in decimal number is too large")
+        this.frac = if (frac.isEmpty()) {
+            0
+        } else {
+            var tmp = frac.toIntOrNull() ?: 
+                throw badRequest("Invalid fractional value")
+            if (tmp > TalerAmount.FRACTION_BASE) 
+                throw badRequest("Fractional value specified in decimal number is too large")
+            repeat(8 - frac.length) {
+                tmp *= 10
+            }
+            tmp
+        }
+    }
+
+    override fun equals(other: Any?): Boolean {
+        return other is DecimalNumber &&
+                other.value == this.value &&
+                other.frac == this.frac
+    }
+
+    override fun toString(): String {
+        return if (frac == 0) {
+            "$value"
+        } else {
+            "$value.${frac.toString().padStart(8, '0')}"
+                .dropLastWhile { it == '0' } // Trim useless fractional trailing 0
+        }
+    }
+
+    internal object Serializer : KSerializer<DecimalNumber> {
+        override val descriptor: SerialDescriptor =
+            PrimitiveSerialDescriptor("DecimalNumber", PrimitiveKind.STRING)
+    
+        override fun serialize(encoder: Encoder, value: DecimalNumber) {
+            encoder.encodeString(value.toString())
+        }
+    
+        override fun deserialize(decoder: Decoder): DecimalNumber {
+            return DecimalNumber(decoder.decodeString())
+        }
+    }
+
+    companion object {
+        private val PATTERN = Regex("([0-9]+)(?:\\.([0-9]{1,8}))?")
+    }
+}
+
 @Serializable(with = TalerAmount.Serializer::class)
 class TalerAmount {
     val value: Long
@@ -149,6 +205,8 @@ class TalerAmount {
         }
     }
 
+    fun number(): DecimalNumber = DecimalNumber(value, frac)
+
     override fun equals(other: Any?): Boolean {
         return other is TalerAmount &&
                 other.value == this.value &&
@@ -157,10 +215,10 @@ class TalerAmount {
     }
 
     override fun toString(): String {
-        if (frac == 0) {
-            return "$currency:$value"
+        return if (frac == 0) {
+            "$currency:$value"
         } else {
-            return "$currency:$value.${frac.toString().padStart(8, '0')}"
+            "$currency:$value.${frac.toString().padStart(8, '0')}"
                 .dropLastWhile { it == '0' } // Trim useless fractional trailing 0
         }
     }
@@ -182,6 +240,9 @@ class TalerAmount {
         const val FRACTION_BASE = 100000000
         const val MAX_VALUE = 4503599627370496L // 2^52
         private val PATTERN = Regex("([A-Z]{1,11}):([0-9]+)(?:\\.([0-9]{1,8}))?")
+
+        fun zero(currency: String) = TalerAmount(0, 0, currency)
+        fun max(currency: String) = TalerAmount(MAX_VALUE, FRACTION_BASE-1, currency)
     }
 }
 
@@ -232,10 +293,7 @@ sealed class Payto {
 
     /** Transform a payto URI to its bank form, using [name] as the receiver-name and the bank [ctx] */
     fun bank(name: String, ctx: BankPaytoCtx): String = when (this) {
-        is IbanPayto -> {
-            val bic = if (ctx.bic != null) "${ctx.bic}/" else ""
-            "payto://iban/$bic$iban?receiver-name=${name.encodeURLParameter()}"
-        }
+        is IbanPayto -> IbanPayto.build(iban.toString(), ctx.bic, name)
         is XTalerBankPayto -> "payto://x-taler-bank/${ctx.hostname ?: "localhost"}/$username?receiver-name=${name.encodeURLParameter()}"
     }
 
@@ -246,6 +304,14 @@ sealed class Payto {
         }
     }
 
+    fun expectRequestIban(): IbanPayto {
+        try {
+            return expectIban()
+        } catch (e: Exception) {
+            throw BadRequestException(e.message ?: "", e)
+        }
+    }
+
     fun expectXTalerBank(): XTalerBankPayto {
         return when (this) {
             is XTalerBankPayto -> this
@@ -253,6 +319,14 @@ sealed class Payto {
         }
     }
 
+    fun expectRequestXTalerBank(): XTalerBankPayto {
+        try {
+            return expectXTalerBank()
+        } catch (e: Exception) {
+            throw BadRequestException(e.message ?: "", e)
+        }
+    }
+
     internal object Serializer : KSerializer<Payto> {
         override val descriptor: SerialDescriptor =
             PrimitiveSerialDescriptor("Payto", PrimitiveKind.STRING)
@@ -275,7 +349,7 @@ sealed class Payto {
             }
             if (parsed.scheme != "payto") throw CommonError.Payto("expect a payto URI got '${parsed.scheme}'")
 
-            val params = (parsed.query ?: "").parseUrlEncodedParameters()
+            val params = parseQueryString(parsed.query ?: "")
             val amount = params["amount"]?.run { TalerAmount(this) }
             val message = params["message"]
             val receiverName = params["receiver-name"]
@@ -333,10 +407,7 @@ class IbanPayto internal constructor(
     override fun toString(): String = parsed.toString()
 
     /** Transform an IBAN payto URI to its full form, using [defaultName] if receiver-name is missing */
-    fun full(defaultName: String): String {
-        val bic = if (this.bic != null) "$bic/" else ""
-        return "payto://iban/$bic$iban?receiver-name=${(receiverName ?: defaultName).encodeURLParameter()}"
-    }
+    fun full(defaultName: String): String = build(iban.toString(), bic, receiverName ?: defaultName)
 
     internal object Serializer : KSerializer<IbanPayto> {
         override val descriptor: SerialDescriptor =
@@ -352,6 +423,12 @@ class IbanPayto internal constructor(
     }
 
     companion object {
+        fun build(iban: String, bic: String?, name: String?): String {
+            val bic = if (bic != null) "$bic/" else ""
+            val name = if (name != null) "?receiver-name=${name.encodeURLParameter()}" else ""
+            return "payto://iban/$bic$iban$name"
+        }
+
         fun rand(): IbanPayto {
             return parse("payto://iban/SANDBOXX/${IBAN.rand()}").expectIban()
         }
@@ -391,7 +468,7 @@ class Base32Crockford32B {
     constructor(encoded: String) {
         val decoded = try {
             Base32Crockford.decode(encoded) 
-        } catch (e: EncodingException) {
+        } catch (e: IllegalArgumentException) {
             null
         }
         require(decoded != null && decoded.size == 32) {
@@ -408,8 +485,9 @@ class Base32Crockford32B {
     }
 
     fun encoded(): String {
-        encoded = encoded ?: Base32Crockford.encode(raw)
-        return encoded!!
+        val tmp = encoded ?: Base32Crockford.encode(raw)
+        encoded = tmp
+        return tmp
     }
 
     override fun toString(): String {
@@ -444,7 +522,7 @@ class Base32Crockford64B {
     constructor(encoded: String) {
         val decoded = try {
             Base32Crockford.decode(encoded) 
-        } catch (e: EncodingException) {
+        } catch (e: IllegalArgumentException) {
             null
         }
         
@@ -462,8 +540,9 @@ class Base32Crockford64B {
     }
 
     fun encoded(): String {
-        encoded = encoded ?: Base32Crockford.encode(raw)
-        return encoded!!
+        val tmp = encoded ?: Base32Crockford.encode(raw)
+        encoded = tmp
+        return tmp
     }
 
     override fun toString(): String {
