@@ -39,7 +39,7 @@ fun Instant.micros(): Long {
         if (micros == Long.MAX_VALUE) throw ArithmeticException()
         return micros
     } catch (e: ArithmeticException) {
-        throw Exception("${this} is too big to be converted to micros resolution", e)
+        throw Exception("$this is too big to be converted to micros resolution", e)
     }
 }
 
@@ -54,33 +54,6 @@ fun Long.asInstant(): Instant {
     return try {
         Instant.EPOCH.plus(this, ChronoUnit.MICROS)
     } catch (e: ArithmeticException ) {
-        throw Exception("${this} is too big to be converted to Instant", e)
+        throw Exception("$this is too big to be converted to Instant", e)
     }
-}
-
-/**
- * Returns the minimum instant between two.
- *
- * @param a input [Instant]
- * @param b input [Instant]
- * @return the minimum [Instant] or null if even one is null.
- */
-fun minTimestamp(a: Instant?, b: Instant?): Instant? {
-    if (a == null || b == null) return null
-    if (a.isBefore(b)) return a
-    return b // includes the case where a == b.
-}
-
-/**
- * Returns the max instant between two.
- *
- * @param a input [Instant]
- * @param b input [Instant]
- * @return the max [Instant] or null if both are null
- */
-fun maxTimestamp(a: Instant?, b: Instant?): Instant? {
-    if (a == null) return b
-    if (b == null) return a
-    if (a.isAfter(b)) return a
-    return b // includes the case where a == b
 }
\ No newline at end of file
