@@ -28,7 +28,7 @@ CREATE FUNCTION amount_normalize(
     IN amount taler_amount
   ,OUT normalized taler_amount
 )
-LANGUAGE plpgsql AS $$
+LANGUAGE plpgsql IMMUTABLE AS $$
 BEGIN
   normalized.val = amount.val + amount.frac / 100000000;
   IF (normalized.val > 1::INT8<<52) THEN
@@ -42,13 +42,13 @@ COMMENT ON FUNCTION amount_normalize
       'It raises an exception when the resulting .val is larger than 2^52';
 
 CREATE FUNCTION amount_add(
-   IN a taler_amount
-  ,IN b taler_amount
+   IN l taler_amount
+  ,IN r taler_amount
   ,OUT sum taler_amount
 )
-LANGUAGE plpgsql AS $$
+LANGUAGE plpgsql IMMUTABLE AS $$
 BEGIN
-  sum = (a.val + b.val, a.frac + b.frac);
+  sum = (l.val + r.val, l.frac + r.frac);
   SELECT normalized.val, normalized.frac INTO sum.val, sum.frac FROM amount_normalize(sum) as normalized;
 END $$;
 COMMENT ON FUNCTION amount_add
@@ -60,7 +60,7 @@ CREATE FUNCTION amount_left_minus_right(
  ,OUT diff taler_amount
  ,OUT ok BOOLEAN
 )
-LANGUAGE plpgsql AS $$
+LANGUAGE plpgsql IMMUTABLE AS $$
 BEGIN
 IF l.val > r.val THEN
   ok = TRUE;
@@ -87,15 +87,38 @@ COMMENT ON FUNCTION amount_left_minus_right
 CREATE FUNCTION account_balance_is_sufficient(
   IN in_account_id INT8,
   IN in_amount taler_amount,
-  OUT out_balance_insufficient BOOLEAN
+  IN in_wire_transfer_fees taler_amount,
+  IN in_min_amount taler_amount,
+  IN in_max_amount taler_amount,
+  OUT out_balance_insufficient BOOLEAN,
+  OUT out_bad_amount BOOLEAN
 )
 LANGUAGE plpgsql AS $$ 
 DECLARE
 account_has_debt BOOLEAN;
 account_balance taler_amount;
 account_max_debt taler_amount;
+amount_with_fee taler_amount;
 BEGIN
--- get account info, we expect the account to exist
+
+-- Check min and max
+SELECT (SELECT in_min_amount IS NOT NULL AND NOT ok FROM amount_left_minus_right(in_amount, in_min_amount)) OR
+  (SELECT in_max_amount IS NOT NULL AND NOT ok FROM amount_left_minus_right(in_max_amount, in_amount))
+  INTO out_bad_amount;
+IF out_bad_amount THEN
+  RETURN;
+END IF;
+
+-- Add fees to the amount
+IF in_wire_transfer_fees IS NOT NULL AND in_wire_transfer_fees != (0, 0)::taler_amount THEN
+  SELECT sum.val, sum.frac 
+    INTO amount_with_fee.val, amount_with_fee.frac 
+    FROM amount_add(in_amount, in_wire_transfer_fees) as sum;
+ELSE
+  amount_with_fee = in_amount;
+END IF;
+
+-- Get account info, we expect the account to exist
 SELECT
   has_debt,
   (balance).val, (balance).frac,
@@ -106,12 +129,12 @@ SELECT
     account_max_debt.val, account_max_debt.frac
   FROM bank_accounts WHERE bank_account_id=in_account_id;
 
--- check enough funds
+-- Check enough funds
 IF account_has_debt THEN 
   -- debt case: simply checking against the max debt allowed.
   SELECT sum.val, sum.frac 
     INTO account_balance.val, account_balance.frac 
-    FROM amount_add(account_balance, in_amount) as sum;
+    FROM amount_add(account_balance, amount_with_fee) as sum;
   SELECT NOT ok
     INTO out_balance_insufficient
     FROM amount_left_minus_right(account_max_debt, account_balance);
@@ -121,14 +144,14 @@ IF account_has_debt THEN
 ELSE -- not a debt account
   SELECT NOT ok
     INTO out_balance_insufficient
-    FROM amount_left_minus_right(account_balance, in_amount);
+    FROM amount_left_minus_right(account_balance, amount_with_fee);
   IF out_balance_insufficient THEN
      -- debtor will switch to debt: determine their new negative balance.
     SELECT
       (diff).val, (diff).frac
       INTO
         account_balance.val, account_balance.frac
-      FROM amount_left_minus_right(in_amount, account_balance);
+      FROM amount_left_minus_right(amount_with_fee, account_balance);
     SELECT NOT ok
       INTO out_balance_insufficient
       FROM amount_left_minus_right(account_max_debt, account_balance);
@@ -140,9 +163,340 @@ END IF;
 END $$;
 COMMENT ON FUNCTION account_balance_is_sufficient IS 'Check if an account have enough fund to transfer an amount.';
 
+CREATE FUNCTION account_max_amount(
+  IN in_account_id INT8,
+  IN in_max_amount taler_amount,
+  OUT out_max_amount taler_amount
+)
+LANGUAGE plpgsql AS $$
+BEGIN
+-- add balance and max_debt
+WITH added AS (
+  SELECT amount_add(balance, max_debt) as sum FROM bank_accounts
+    WHERE bank_account_id=in_account_id
+) SELECT (sum).val, (sum).frac
+  INTO out_max_amount.val, out_max_amount.frac
+  FROM added;
+
+IF in_max_amount.val < out_max_amount.val 
+  OR (in_max_amount.val = out_max_amount.val OR in_max_amount.frac < out_max_amount.frac) THEN
+  out_max_amount = in_max_amount;
+END IF;
+END $$;
+
+CREATE FUNCTION bank_wire_transfer(
+  IN in_creditor_account_id INT8,
+  IN in_debtor_account_id INT8,
+  IN in_subject TEXT,
+  IN in_amount taler_amount,
+  IN in_timestamp INT8,
+  IN in_wire_transfer_fees taler_amount,
+  IN in_min_amount taler_amount,
+  IN in_max_amount taler_amount,
+  -- Error status
+  OUT out_balance_insufficient BOOLEAN,
+  OUT out_bad_amount BOOLEAN,
+  -- Success return
+  OUT out_credit_row_id INT8,
+  OUT out_debit_row_id INT8
+)
+LANGUAGE plpgsql AS $$
+DECLARE
+amount_with_fee taler_amount;
+admin_account_id INT8;
+admin_has_debt BOOLEAN;
+admin_balance taler_amount;
+admin_payto TEXT;
+admin_name TEXT;
+debtor_has_debt BOOLEAN;
+debtor_balance taler_amount;
+debtor_max_debt taler_amount;
+debtor_payto TEXT;
+debtor_name TEXT;
+creditor_has_debt BOOLEAN;
+creditor_balance taler_amount;
+creditor_payto TEXT;
+creditor_name TEXT;
+tmp_balance taler_amount;
+BEGIN
+
+-- Check min and max
+SELECT (SELECT in_min_amount IS NOT NULL AND NOT ok FROM amount_left_minus_right(in_amount, in_min_amount)) OR
+  (SELECT in_max_amount IS NOT NULL AND NOT ok FROM amount_left_minus_right(in_max_amount, in_amount))
+  INTO out_bad_amount;
+IF out_bad_amount THEN
+  RETURN;
+END IF;
+
+-- Retrieve debtor info
+SELECT
+  has_debt,
+  (balance).val, (balance).frac,
+  (max_debt).val, (max_debt).frac,
+  internal_payto_uri, customers.name
+  INTO
+    debtor_has_debt,
+    debtor_balance.val, debtor_balance.frac,
+    debtor_max_debt.val, debtor_max_debt.frac,
+    debtor_payto, debtor_name
+  FROM bank_accounts
+    JOIN customers ON customer_id=owning_customer_id
+  WHERE bank_account_id=in_debtor_account_id;
+IF NOT FOUND THEN
+  RAISE EXCEPTION 'Unknown debtor %', in_debtor_account_id;
+END IF;
+-- Retrieve creditor info
+SELECT
+  has_debt,
+  (balance).val, (balance).frac,
+  internal_payto_uri, customers.name
+  INTO
+    creditor_has_debt,
+    creditor_balance.val, creditor_balance.frac,
+    creditor_payto, creditor_name
+  FROM bank_accounts
+    JOIN customers ON customer_id=owning_customer_id
+  WHERE bank_account_id=in_creditor_account_id;
+IF NOT FOUND THEN
+  RAISE EXCEPTION 'Unknown creditor %', in_creditor_account_id;
+END IF;
+-- Retrieve admin info
+SELECT
+  bank_account_id, has_debt,
+  (balance).val, (balance).frac,
+  internal_payto_uri, customers.name
+  INTO 
+    admin_account_id, admin_has_debt,
+    admin_balance.val, admin_balance.frac,
+    admin_payto, admin_name
+  FROM bank_accounts
+    JOIN customers ON customer_id=owning_customer_id
+  WHERE login = 'admin';
+IF NOT FOUND THEN
+  RAISE EXCEPTION 'No admin';
+END IF;
+
+-- Add fees to the amount
+IF in_wire_transfer_fees IS NOT NULL AND in_wire_transfer_fees != (0, 0)::taler_amount AND admin_account_id != in_debtor_account_id THEN
+  SELECT sum.val, sum.frac 
+    INTO amount_with_fee.val, amount_with_fee.frac 
+    FROM amount_add(in_amount, in_wire_transfer_fees) as sum;
+ELSE
+  amount_with_fee = in_amount;
+END IF;
+
+-- DEBTOR SIDE
+-- check debtor has enough funds.
+IF debtor_has_debt THEN 
+  -- debt case: simply checking against the max debt allowed.
+  SELECT sum.val, sum.frac 
+    INTO debtor_balance.val, debtor_balance.frac 
+    FROM amount_add(debtor_balance, amount_with_fee) as sum;
+  SELECT NOT ok
+    INTO out_balance_insufficient
+    FROM amount_left_minus_right(debtor_max_debt,
+                                 debtor_balance);
+  IF out_balance_insufficient THEN
+    RETURN;
+  END IF;
+ELSE -- not a debt account
+  SELECT
+    NOT ok,
+    (diff).val, (diff).frac
+    INTO
+      out_balance_insufficient,
+      tmp_balance.val,
+      tmp_balance.frac
+    FROM amount_left_minus_right(debtor_balance,
+                                 amount_with_fee);
+  IF NOT out_balance_insufficient THEN -- debtor has enough funds in the (positive) balance.
+    debtor_balance=tmp_balance;
+  ELSE -- debtor will switch to debt: determine their new negative balance.
+    SELECT
+      (diff).val, (diff).frac
+      INTO
+        debtor_balance.val, debtor_balance.frac
+      FROM amount_left_minus_right(amount_with_fee,
+                                   debtor_balance);
+    debtor_has_debt=TRUE;
+    SELECT NOT ok
+      INTO out_balance_insufficient
+      FROM amount_left_minus_right(debtor_max_debt,
+                                   debtor_balance);
+    IF out_balance_insufficient THEN
+      RETURN;
+    END IF;
+  END IF;
+END IF;
+
+-- CREDITOR SIDE.
+-- Here we figure out whether the creditor would switch
+-- from debit to a credit situation, and adjust the balance
+-- accordingly.
+IF NOT creditor_has_debt THEN -- easy case.
+  SELECT sum.val, sum.frac 
+    INTO creditor_balance.val, creditor_balance.frac 
+    FROM amount_add(creditor_balance, in_amount) as sum;
+ELSE -- creditor had debit but MIGHT switch to credit.
+  SELECT
+    (diff).val, (diff).frac,
+    NOT ok
+    INTO
+      tmp_balance.val, tmp_balance.frac,
+      creditor_has_debt
+    FROM amount_left_minus_right(in_amount,
+                                 creditor_balance);
+  IF NOT creditor_has_debt THEN
+    creditor_balance=tmp_balance;
+  ELSE
+    -- the amount is not enough to bring the receiver
+    -- to a credit state, switch operators to calculate the new balance.
+    SELECT
+      (diff).val, (diff).frac
+      INTO creditor_balance.val, creditor_balance.frac
+      FROM amount_left_minus_right(creditor_balance,
+	                           in_amount);
+  END IF;
+END IF;
+
+-- ADMIN SIDE.
+-- Here we figure out whether the administrator would switch
+-- from debit to a credit situation, and adjust the balance
+-- accordingly.
+IF amount_with_fee != in_amount THEN
+  IF NOT admin_has_debt THEN -- easy case.
+    SELECT sum.val, sum.frac 
+      INTO admin_balance.val, admin_balance.frac 
+      FROM amount_add(admin_balance, in_wire_transfer_fees) as sum;
+  ELSE -- creditor had debit but MIGHT switch to credit.
+    SELECT (diff).val, (diff).frac, NOT ok
+      INTO
+        tmp_balance.val, tmp_balance.frac,
+        admin_has_debt
+      FROM amount_left_minus_right(in_wire_transfer_fees, admin_balance);
+    IF NOT admin_has_debt THEN
+      admin_balance=tmp_balance;
+    ELSE
+      -- the amount is not enough to bring the receiver
+      -- to a credit state, switch operators to calculate the new balance.
+      SELECT (diff).val, (diff).frac
+        INTO admin_balance.val, admin_balance.frac
+        FROM amount_left_minus_right(admin_balance, in_wire_transfer_fees);
+    END IF;
+  END IF;
+END IF;
+
+-- now actually create the bank transaction.
+-- debtor side:
+INSERT INTO bank_account_transactions (
+  creditor_payto_uri
+  ,creditor_name
+  ,debtor_payto_uri
+  ,debtor_name
+  ,subject
+  ,amount
+  ,transaction_date
+  ,direction
+  ,bank_account_id
+  )
+VALUES (
+  creditor_payto,
+  creditor_name,
+  debtor_payto,
+  debtor_name,
+  in_subject,
+  in_amount,
+  in_timestamp,
+  'debit',
+  in_debtor_account_id
+) RETURNING bank_transaction_id INTO out_debit_row_id;
+
+-- debtor side:
+INSERT INTO bank_account_transactions (
+  creditor_payto_uri
+  ,creditor_name
+  ,debtor_payto_uri
+  ,debtor_name
+  ,subject
+  ,amount
+  ,transaction_date
+  ,direction
+  ,bank_account_id
+  )
+VALUES (
+  creditor_payto,
+  creditor_name,
+  debtor_payto,
+  debtor_name,
+  in_subject,
+  in_amount,
+  in_timestamp,
+  'credit',
+  in_creditor_account_id
+) RETURNING bank_transaction_id INTO out_credit_row_id;
+
+-- checks and balances set up, now update bank accounts.
+UPDATE bank_accounts
+SET
+  balance=debtor_balance,
+  has_debt=debtor_has_debt
+WHERE bank_account_id=in_debtor_account_id;
+
+UPDATE bank_accounts
+SET
+  balance=creditor_balance,
+  has_debt=creditor_has_debt
+WHERE bank_account_id=in_creditor_account_id;
+
+-- Fee part
+IF amount_with_fee != in_amount THEN
+  INSERT INTO bank_account_transactions (
+    creditor_payto_uri
+    ,creditor_name
+    ,debtor_payto_uri
+    ,debtor_name
+    ,subject
+    ,amount
+    ,transaction_date
+    ,direction
+    ,bank_account_id
+    )
+  VALUES (
+    admin_payto,
+    admin_name,
+    debtor_payto,
+    debtor_name,
+    'wire transfer fees for tx ' || out_debit_row_id,
+    in_wire_transfer_fees,
+    in_timestamp,
+    'debit',
+    in_debtor_account_id
+  ), (
+    admin_payto,
+    admin_name,
+    debtor_payto,
+    debtor_name,
+    'wire transfer fees for tx ' || out_debit_row_id,
+    in_amount,
+    in_timestamp,
+    'credit',
+    admin_account_id
+  );
+
+  UPDATE bank_accounts
+  SET
+    balance=admin_balance,
+    has_debt=admin_has_debt
+  WHERE bank_account_id=admin_account_id;
+END IF;
+
+-- notify new transaction
+PERFORM pg_notify('bank_tx', in_debtor_account_id || ' ' || in_creditor_account_id || ' ' || out_debit_row_id || ' ' || out_credit_row_id);
+END $$;
+
 CREATE FUNCTION account_delete(
   IN in_login TEXT,
-  IN in_now INT8,
+  IN in_timestamp INT8,
   IN in_is_tan BOOLEAN,
   OUT out_not_found BOOLEAN,
   OUT out_balance_not_zero BOOLEAN,
@@ -170,7 +524,7 @@ IF NOT FOUND OR out_balance_not_zero OR out_tan_required THEN
 END IF;
 
 -- actual deletion
-UPDATE customers SET deleted_at = in_now WHERE customer_id = my_customer_id;
+UPDATE customers SET deleted_at = in_timestamp WHERE customer_id = my_customer_id;
 END $$;
 COMMENT ON FUNCTION account_delete IS 'Deletes an account if the balance is zero';
 
@@ -418,6 +772,9 @@ CREATE FUNCTION bank_transaction(
   IN in_timestamp INT8,
   IN in_is_tan BOOLEAN,
   IN in_request_uid BYTEA,
+  IN in_wire_transfer_fees taler_amount,
+  IN in_min_amount taler_amount,
+  IN in_max_amount taler_amount,
   -- Error status
   OUT out_creditor_not_found BOOLEAN,
   OUT out_debtor_not_found BOOLEAN,
@@ -426,6 +783,7 @@ CREATE FUNCTION bank_transaction(
   OUT out_creditor_admin BOOLEAN,
   OUT out_tan_required BOOLEAN,
   OUT out_request_uid_reuse BOOLEAN,
+  OUT out_bad_amount BOOLEAN,
   -- Success return
   OUT out_credit_bank_account_id INT8,
   OUT out_debit_bank_account_id INT8,
@@ -477,10 +835,12 @@ END IF;
 -- Perform bank transfer
 SELECT
   transfer.out_balance_insufficient,
+  transfer.out_bad_amount,
   transfer.out_credit_row_id,
   transfer.out_debit_row_id
   INTO
     out_balance_insufficient,
+    out_bad_amount,
     out_credit_row_id,
     out_debit_row_id
   FROM bank_wire_transfer(
@@ -489,10 +849,14 @@ SELECT
     in_subject,
     in_amount,
     in_timestamp,
-    NULL,
-    NULL,
-    NULL
+    in_wire_transfer_fees,
+    in_min_amount,
+    in_max_amount
   ) as transfer;
+IF out_balance_insufficient OR out_bad_amount THEN
+  RETURN;
+END IF;
+
 -- Store operation
 IF in_request_uid IS NOT NULL THEN
   INSERT INTO bank_transaction_operations (request_uid, bank_transaction)  
@@ -505,15 +869,21 @@ CREATE FUNCTION create_taler_withdrawal(
   IN in_account_username TEXT,
   IN in_withdrawal_uuid UUID,
   IN in_amount taler_amount,
-  IN in_now_date INT8,
+  IN in_suggested_amount taler_amount,
+  IN in_timestamp INT8,
+  IN in_wire_transfer_fees taler_amount,
+  IN in_min_amount taler_amount,
+  IN in_max_amount taler_amount,
    -- Error status
   OUT out_account_not_found BOOLEAN,
   OUT out_account_is_exchange BOOLEAN,
-  OUT out_balance_insufficient BOOLEAN
+  OUT out_balance_insufficient BOOLEAN,
+  OUT out_bad_amount BOOLEAN
 )
 LANGUAGE plpgsql AS $$ 
 DECLARE
 account_id INT8;
+amount_with_fee taler_amount;
 BEGIN
 -- Check account exists
 SELECT bank_account_id, is_taler_exchange
@@ -527,15 +897,23 @@ IF NOT FOUND OR out_account_is_exchange THEN
 END IF;
 
 -- Check enough funds
-SELECT account_balance_is_sufficient(account_id, in_amount) INTO out_balance_insufficient;
-IF out_balance_insufficient THEN
-  RETURN;
+IF in_amount IS NOT NULL OR in_suggested_amount IS NOT NULL THEN
+  SELECT test.out_balance_insufficient, test.out_bad_amount FROM account_balance_is_sufficient(
+    account_id, 
+    COALESCE(in_amount, in_suggested_amount), 
+    in_wire_transfer_fees,
+    in_min_amount,
+    in_max_amount
+  ) AS test INTO out_balance_insufficient, out_bad_amount;
+  IF out_balance_insufficient OR out_bad_amount THEN
+    RETURN;
+  END IF;
 END IF;
 
 -- Create withdrawal operation
 INSERT INTO taler_withdrawal_operations
-    (withdrawal_uuid, wallet_bank_account, amount, creation_date)
-  VALUES (in_withdrawal_uuid, account_id, in_amount, in_now_date);
+    (withdrawal_uuid, wallet_bank_account, amount, suggested_amount, creation_date)
+  VALUES (in_withdrawal_uuid, account_id, in_amount, in_suggested_amount, in_timestamp);
 END $$;
 COMMENT ON FUNCTION create_taler_withdrawal IS 'Create a new withdrawal operation';
 
@@ -544,18 +922,28 @@ CREATE FUNCTION select_taler_withdrawal(
   IN in_reserve_pub BYTEA,
   IN in_subject TEXT,
   IN in_selected_exchange_payto TEXT,
+  IN in_amount taler_amount,
+  IN in_wire_transfer_fees taler_amount,
+  IN in_min_amount taler_amount,
+  IN in_max_amount taler_amount,
   -- Error status
   OUT out_no_op BOOLEAN,
   OUT out_already_selected BOOLEAN,
   OUT out_reserve_pub_reuse BOOLEAN,
   OUT out_account_not_found BOOLEAN,
   OUT out_account_is_not_exchange BOOLEAN,
+  OUT out_missing_amount BOOLEAN,
+  OUT out_amount_differs BOOLEAN,
+  OUT out_balance_insufficient BOOLEAN,
+  OUT out_bad_amount BOOLEAN,
   -- Success return
   OUT out_status TEXT
 )
 LANGUAGE plpgsql AS $$ 
 DECLARE
 not_selected BOOLEAN;
+account_id int8;
+amount_with_fee taler_amount;
 BEGIN
 -- Check for conflict and idempotence
 SELECT
@@ -566,11 +954,14 @@ SELECT
     ELSE 'selected'
   END,
   selection_done 
-    AND (selected_exchange_payto != in_selected_exchange_payto OR reserve_pub != in_reserve_pub)
-  INTO not_selected, out_status, out_already_selected
+    AND (selected_exchange_payto != in_selected_exchange_payto OR reserve_pub != in_reserve_pub OR amount != in_amount),
+  amount IS NULL AND in_amount IS NULL,
+  amount IS NOT NULL AND amount != in_amount,
+  wallet_bank_account
+  INTO not_selected, out_status, out_already_selected, out_missing_amount, out_amount_differs, account_id
   FROM taler_withdrawal_operations
   WHERE withdrawal_uuid=in_withdrawal_uuid;
-IF NOT FOUND OR out_already_selected THEN
+IF NOT FOUND OR out_already_selected OR out_missing_amount OR out_amount_differs THEN
   out_no_op=NOT FOUND;
   RETURN;
 END IF;
@@ -584,6 +975,7 @@ IF not_selected THEN
   IF out_reserve_pub_reuse THEN
     RETURN;
   END IF;
+
   -- Check exchange account
   SELECT NOT is_taler_exchange
     INTO out_account_is_not_exchange
@@ -594,9 +986,26 @@ IF not_selected THEN
     RETURN;
   END IF;
 
+  IF in_amount IS NOT NULL THEN
+    SELECT test.out_balance_insufficient, test.out_bad_amount FROM account_balance_is_sufficient(
+      account_id,
+      in_amount,
+      in_wire_transfer_fees,
+      in_min_amount,
+      in_max_amount
+    ) AS test INTO out_balance_insufficient, out_bad_amount;
+    IF out_balance_insufficient OR out_bad_amount THEN
+      RETURN;
+    END IF;
+  END IF;
+
   -- Update withdrawal operation
   UPDATE taler_withdrawal_operations
-    SET selected_exchange_payto=in_selected_exchange_payto, reserve_pub=in_reserve_pub, subject=in_subject, selection_done=true
+    SET selected_exchange_payto=in_selected_exchange_payto,
+        reserve_pub=in_reserve_pub,
+        subject=in_subject,
+        selection_done=true,
+        amount=COALESCE(amount, in_amount)
     WHERE withdrawal_uuid=in_withdrawal_uuid;
 
   -- Notify status change
@@ -630,10 +1039,14 @@ COMMENT ON FUNCTION abort_taler_withdrawal IS 'Abort a withdrawal operation.';
 CREATE FUNCTION confirm_taler_withdrawal(
   IN in_login TEXT,
   IN in_withdrawal_uuid uuid,
-  IN in_confirmation_date INT8,
+  IN in_timestamp INT8,
   IN in_is_tan BOOLEAN,
+  IN in_wire_transfer_fees taler_amount,
+  IN in_min_amount taler_amount,
+  IN in_max_amount taler_amount,
   OUT out_no_op BOOLEAN,
   OUT out_balance_insufficient BOOLEAN,
+  OUT out_bad_amount BOOLEAN,
   OUT out_creditor_not_found BOOLEAN,
   OUT out_exchange_not_found BOOLEAN,
   OUT out_not_selected BOOLEAN,
@@ -677,7 +1090,7 @@ IF NOT FOUND OR already_confirmed OR out_aborted OR out_not_selected THEN
   RETURN;
 END IF;
 
--- Check exchange account then 2faa
+-- Check exchange account then 2fa
 SELECT
   bank_account_id
   INTO exchange_bank_account_id
@@ -690,19 +1103,20 @@ END IF;
 
 SELECT -- not checking for accounts existence, as it was done above.
   transfer.out_balance_insufficient,
+  transfer.out_bad_amount,
   out_credit_row_id
-  INTO out_balance_insufficient, tx_row_id
+  INTO out_balance_insufficient, out_bad_amount, tx_row_id
 FROM bank_wire_transfer(
   exchange_bank_account_id,
   wallet_bank_account_local,
   subject_local,
   amount_local,
-  in_confirmation_date,
-  NULL,
-  NULL,
-  NULL
+  in_timestamp,
+  in_wire_transfer_fees,
+  in_min_amount,
+  in_max_amount
 ) as transfer;
-IF out_balance_insufficient THEN
+IF out_balance_insufficient OR out_bad_amount THEN
   RETURN;
 END IF;
 
@@ -720,229 +1134,8 @@ END $$;
 COMMENT ON FUNCTION confirm_taler_withdrawal
   IS 'Set a withdrawal operation as confirmed and wire the funds to the exchange.';
 
-CREATE FUNCTION bank_wire_transfer(
-  IN in_creditor_account_id INT8,
-  IN in_debtor_account_id INT8,
-  IN in_subject TEXT,
-  IN in_amount taler_amount,
-  IN in_transaction_date INT8,
-  IN in_account_servicer_reference TEXT,
-  IN in_payment_information_id TEXT,
-  IN in_end_to_end_id TEXT,
-  -- Error status
-  OUT out_balance_insufficient BOOLEAN,
-  -- Success return
-  OUT out_credit_row_id INT8,
-  OUT out_debit_row_id INT8
-)
-LANGUAGE plpgsql AS $$
-DECLARE
-debtor_has_debt BOOLEAN;
-debtor_balance taler_amount;
-debtor_max_debt taler_amount;
-debtor_payto_uri TEXT;
-debtor_name TEXT;
-creditor_has_debt BOOLEAN;
-creditor_balance taler_amount;
-creditor_payto_uri TEXT;
-creditor_name TEXT;
-potential_balance taler_amount;
-new_debtor_balance taler_amount;
-new_debtor_balance_ok BOOLEAN;
-new_creditor_balance taler_amount;
-will_debtor_have_debt BOOLEAN;
-will_creditor_have_debt BOOLEAN;
-BEGIN
--- Retrieve debtor info
-SELECT
-  has_debt,
-  (balance).val, (balance).frac,
-  (max_debt).val, (max_debt).frac,
-  internal_payto_uri, customers.name
-  INTO
-    debtor_has_debt,
-    debtor_balance.val, debtor_balance.frac,
-    debtor_max_debt.val, debtor_max_debt.frac,
-    debtor_payto_uri, debtor_name
-  FROM bank_accounts
-    JOIN customers ON customer_id=owning_customer_id
-  WHERE bank_account_id=in_debtor_account_id;
-IF NOT FOUND THEN
-  RAISE EXCEPTION 'fuck debtor';
-END IF;
--- Retrieve creditor info
-SELECT
-  has_debt,
-  (balance).val, (balance).frac,
-  internal_payto_uri, customers.name
-  INTO
-    creditor_has_debt,
-    creditor_balance.val, creditor_balance.frac,
-    creditor_payto_uri, creditor_name
-  FROM bank_accounts
-    JOIN customers ON customer_id=owning_customer_id
-  WHERE bank_account_id=in_creditor_account_id;
-IF NOT FOUND THEN
-  RAISE EXCEPTION 'fuck creditor %', in_creditor_account_id;
-END IF;
-
--- DEBTOR SIDE
--- check debtor has enough funds.
-IF debtor_has_debt THEN 
-  -- debt case: simply checking against the max debt allowed.
-  SELECT sum.val, sum.frac 
-    INTO potential_balance.val, potential_balance.frac 
-    FROM amount_add(debtor_balance, in_amount) as sum;
-  SELECT NOT ok
-    INTO out_balance_insufficient
-    FROM amount_left_minus_right(debtor_max_debt,
-                                 potential_balance);
-  IF out_balance_insufficient THEN
-    RETURN;
-  END IF;
-  new_debtor_balance=potential_balance;
-  will_debtor_have_debt=TRUE;
-ELSE -- not a debt account
-  SELECT
-    NOT ok,
-    (diff).val, (diff).frac
-    INTO
-      out_balance_insufficient,
-      potential_balance.val,
-      potential_balance.frac
-    FROM amount_left_minus_right(debtor_balance,
-                                 in_amount);
-  IF NOT out_balance_insufficient THEN -- debtor has enough funds in the (positive) balance.
-    new_debtor_balance=potential_balance;
-    will_debtor_have_debt=FALSE;
-  ELSE -- debtor will switch to debt: determine their new negative balance.
-    SELECT
-      (diff).val, (diff).frac
-      INTO
-        new_debtor_balance.val, new_debtor_balance.frac
-      FROM amount_left_minus_right(in_amount,
-                                   debtor_balance);
-    will_debtor_have_debt=TRUE;
-    SELECT NOT ok
-      INTO out_balance_insufficient
-      FROM amount_left_minus_right(debtor_max_debt,
-                                   new_debtor_balance);
-    IF out_balance_insufficient THEN
-      RETURN;
-    END IF;
-  END IF;
-END IF;
-out_balance_insufficient=FALSE;
-
--- CREDITOR SIDE.
--- Here we figure out whether the creditor would switch
--- from debit to a credit situation, and adjust the balance
--- accordingly.
-IF NOT creditor_has_debt THEN -- easy case.
-  SELECT sum.val, sum.frac 
-    INTO new_creditor_balance.val, new_creditor_balance.frac 
-    FROM amount_add(creditor_balance, in_amount) as sum;
-  will_creditor_have_debt=FALSE;
-ELSE -- creditor had debit but MIGHT switch to credit.
-  SELECT
-    (diff).val, (diff).frac,
-    NOT ok
-    INTO
-      new_creditor_balance.val, new_creditor_balance.frac,
-      will_creditor_have_debt
-    FROM amount_left_minus_right(in_amount,
-                                 creditor_balance);
-  IF will_creditor_have_debt THEN
-    -- the amount is not enough to bring the receiver
-    -- to a credit state, switch operators to calculate the new balance.
-    SELECT
-      (diff).val, (diff).frac
-      INTO new_creditor_balance.val, new_creditor_balance.frac
-      FROM amount_left_minus_right(creditor_balance,
-	                           in_amount);
-  END IF;
-END IF;
-
--- now actually create the bank transaction.
--- debtor side:
-INSERT INTO bank_account_transactions (
-  creditor_payto_uri
-  ,creditor_name
-  ,debtor_payto_uri
-  ,debtor_name
-  ,subject
-  ,amount
-  ,transaction_date
-  ,account_servicer_reference
-  ,payment_information_id
-  ,end_to_end_id
-  ,direction
-  ,bank_account_id
-  )
-VALUES (
-  creditor_payto_uri,
-  creditor_name,
-  debtor_payto_uri,
-  debtor_name,
-  in_subject,
-  in_amount,
-  in_transaction_date,
-  in_account_servicer_reference,
-  in_payment_information_id,
-  in_end_to_end_id,
-  'debit',
-  in_debtor_account_id
-) RETURNING bank_transaction_id INTO out_debit_row_id;
-
--- debtor side:
-INSERT INTO bank_account_transactions (
-  creditor_payto_uri
-  ,creditor_name
-  ,debtor_payto_uri
-  ,debtor_name
-  ,subject
-  ,amount
-  ,transaction_date
-  ,account_servicer_reference
-  ,payment_information_id
-  ,end_to_end_id
-  ,direction
-  ,bank_account_id
-  )
-VALUES (
-  creditor_payto_uri,
-  creditor_name,
-  debtor_payto_uri,
-  debtor_name,
-  in_subject,
-  in_amount,
-  in_transaction_date,
-  in_account_servicer_reference,
-  in_payment_information_id,
-  in_end_to_end_id, -- does this interest the receiving party?
-  'credit',
-  in_creditor_account_id
-) RETURNING bank_transaction_id INTO out_credit_row_id;
-
--- checks and balances set up, now update bank accounts.
-UPDATE bank_accounts
-SET
-  balance=new_debtor_balance,
-  has_debt=will_debtor_have_debt
-WHERE bank_account_id=in_debtor_account_id;
-
-UPDATE bank_accounts
-SET
-  balance=new_creditor_balance,
-  has_debt=will_creditor_have_debt
-WHERE bank_account_id=in_creditor_account_id;
-
--- notify new transaction
-PERFORM pg_notify('bank_tx', in_debtor_account_id || ' ' || in_creditor_account_id || ' ' || out_debit_row_id || ' ' || out_credit_row_id);
-END $$;
-
 CREATE FUNCTION cashin(
-  IN in_now_date INT8,
+  IN in_timestamp INT8,
   IN in_reserve_pub BYTEA,
   IN in_amount taler_amount,
   IN in_subject TEXT,
@@ -1001,7 +1194,7 @@ SELECT
     admin_account_id,
     in_subject,
     converted_amount,
-    in_now_date,
+    in_timestamp,
     NULL,
     NULL,
     NULL
@@ -1026,7 +1219,7 @@ CREATE FUNCTION cashout_create(
   IN in_amount_debit taler_amount,
   IN in_amount_credit taler_amount,
   IN in_subject TEXT,
-  IN in_now_date INT8,
+  IN in_timestamp INT8,
   IN in_is_tan BOOLEAN,
   -- Error status
   OUT out_bad_conversion BOOLEAN,
@@ -1067,7 +1260,7 @@ ELSIF out_account_is_exchange OR out_no_cashout_payto THEN
   RETURN;
 END IF;
 
--- check conversion TODO use custom min
+-- check conversion
 IF custom_min_cashout.val IS NULL THEN
   custom_min_cashout = NULL;
 END IF;
@@ -1106,7 +1299,7 @@ FROM bank_wire_transfer(
   account_id,
   in_subject,
   in_amount_debit,
-  in_now_date,
+  in_timestamp,
   NULL,
   NULL,
   NULL
@@ -1128,7 +1321,7 @@ INSERT INTO cashout_operations (
   in_request_uid
   ,in_amount_debit
   ,in_amount_credit
-  ,in_now_date
+  ,in_timestamp
   ,account_id
   ,in_subject
   ,tx_id
@@ -1142,7 +1335,7 @@ CREATE FUNCTION tan_challenge_create (
   IN in_body TEXT,
   IN in_op op_enum,
   IN in_code TEXT,
-  IN in_now_date INT8,
+  IN in_timestamp INT8,
   IN in_validity_period INT8,
   IN in_retry_counter INT4,
   IN in_login TEXT,
@@ -1171,8 +1364,8 @@ INSERT INTO tan_challenges (
   in_body,
   in_op,
   in_code,
-  in_now_date,
-  in_now_date + in_validity_period,
+  in_timestamp,
+  in_timestamp + in_validity_period,
   in_retry_counter,
   account_id,
   in_tan_channel,
@@ -1185,7 +1378,7 @@ CREATE FUNCTION tan_challenge_send (
   IN in_challenge_id INT8,
   IN in_login TEXT,
   IN in_code TEXT,              -- New code to use if the old code expired
-  IN in_now_date INT8,        
+  IN in_timestamp INT8,        
   IN in_validity_period INT8,
   IN in_retry_counter INT4,
   -- Error status
@@ -1211,8 +1404,8 @@ FROM customers WHERE login = in_login AND deleted_at IS NULL;
 
 -- Recover expiration date
 SELECT 
-  (in_now_date >= expiration_date OR retry_counter <= 0) AND confirmation_date IS NULL
-  ,in_now_date >= retransmission_date AND confirmation_date IS NULL
+  (in_timestamp >= expiration_date OR retry_counter <= 0) AND confirmation_date IS NULL
+  ,in_timestamp >= retransmission_date AND confirmation_date IS NULL
   ,code, COALESCE(tan_channel, out_tan_channel), COALESCE(tan_info, out_tan_info)
 INTO expired, retransmit, out_tan_code, out_tan_channel, out_tan_info
 FROM tan_challenges WHERE challenge_id = in_challenge_id AND customer = account_id;
@@ -1224,7 +1417,7 @@ END IF;
 IF expired THEN
   UPDATE tan_challenges SET
      code = in_code
-    ,expiration_date = in_now_date + in_validity_period
+    ,expiration_date = in_timestamp + in_validity_period
     ,retry_counter = in_retry_counter
   WHERE challenge_id = in_challenge_id;
   out_tan_code = in_code;
@@ -1236,12 +1429,12 @@ COMMENT ON FUNCTION tan_challenge_send IS 'Get the challenge to send, return NUL
 
 CREATE FUNCTION tan_challenge_mark_sent (
   IN in_challenge_id INT8,
-  IN in_now_date INT8,
+  IN in_timestamp INT8,
   IN in_retransmission_period INT8
 ) RETURNS void
 LANGUAGE sql AS $$
   UPDATE tan_challenges SET 
-    retransmission_date = in_now_date + in_retransmission_period
+    retransmission_date = in_timestamp + in_retransmission_period
   WHERE challenge_id = in_challenge_id;
 $$;
 COMMENT ON FUNCTION tan_challenge_mark_sent IS 'Register a challenge as successfully sent';
@@ -1250,7 +1443,7 @@ CREATE FUNCTION tan_challenge_try (
   IN in_challenge_id INT8, 
   IN in_login TEXT,
   IN in_code TEXT,    
-  IN in_now_date INT8,
+  IN in_timestamp INT8,
   -- Error status       
   OUT out_ok BOOLEAN,
   OUT out_no_op BOOLEAN,
@@ -1271,7 +1464,7 @@ SELECT customer_id INTO account_id FROM customers WHERE login = in_login AND del
 -- Check challenge
 UPDATE tan_challenges SET 
   confirmation_date = CASE 
-    WHEN (retry_counter > 0 AND in_now_date < expiration_date AND code = in_code) THEN in_now_date
+    WHEN (retry_counter > 0 AND in_timestamp < expiration_date AND code = in_code) THEN in_timestamp
     ELSE confirmation_date
   END,
   retry_counter = retry_counter - 1
@@ -1279,7 +1472,7 @@ WHERE challenge_id = in_challenge_id AND customer = account_id
 RETURNING 
   confirmation_date IS NOT NULL, 
   retry_counter <= 0 AND confirmation_date IS NULL,
-  in_now_date >= expiration_date AND confirmation_date IS NULL
+  in_timestamp >= expiration_date AND confirmation_date IS NULL
 INTO out_ok, out_no_retry, out_expired;
 IF NOT FOUND OR NOT out_ok OR out_no_retry OR out_expired THEN
   out_no_op = NOT FOUND;
