@@ -228,15 +228,28 @@ CREATE FUNCTION register_incoming_and_talerable(
   ,OUT out_tx_id INT8
 )
 LANGUAGE plpgsql AS $$
+DECLARE 
+need_reconcile BOOLEAN;
 BEGIN
--- Check conflict
-IF EXISTS (
-  SELECT FROM talerable_incoming_transactions 
-  JOIN incoming_transactions USING(incoming_transaction_id)
-  WHERE reserve_public_key = in_reserve_public_key
-  AND bank_id != in_bank_id
-) THEN
-  out_reserve_pub_reuse = TRUE;
+-- Check if exists
+SELECT incoming_transaction_id, 
+    bank_id IS DISTINCT FROM in_bank_id, 
+    bank_id IS NULL AND amount = in_amount 
+                    AND debit_payto_uri = in_debit_payto_uri
+                    AND wire_transfer_subject = in_wire_transfer_subject
+  INTO out_tx_id, out_reserve_pub_reuse, need_reconcile
+  FROM talerable_incoming_transactions 
+  JOIN incoming_transactions USING(incoming_transaction_id) 
+  WHERE reserve_public_key = in_reserve_public_key;
+
+IF FOUND THEN
+  IF need_reconcile THEN
+    IF in_bank_id IS NOT NULL THEN
+      -- Update the bank_id now that we have it
+      UPDATE incoming_transactions SET bank_id = in_bank_id WHERE incoming_transaction_id = out_tx_id;
+    END IF;
+    out_reserve_pub_reuse=false;
+  END IF;
   RETURN;
 END IF;
 
