@@ -26,11 +26,13 @@ import io.ktor.server.response.*
 import io.ktor.server.routing.*
 import io.ktor.util.pipeline.*
 import tech.libeufin.common.*
-import tech.libeufin.nexus.*
-import tech.libeufin.nexus.db.*
-import tech.libeufin.nexus.db.PaymentDAO.*
-import tech.libeufin.nexus.db.InitiatedDAO.*
-import tech.libeufin.nexus.db.ExchangeDAO.*
+import tech.libeufin.nexus.IncomingPayment
+import tech.libeufin.nexus.NexusConfig
+import tech.libeufin.nexus.checkCurrency
+import tech.libeufin.nexus.db.Database
+import tech.libeufin.nexus.db.ExchangeDAO
+import tech.libeufin.nexus.db.ExchangeDAO.TransferResult
+import tech.libeufin.nexus.db.PaymentDAO.IncomingRegistrationResult
 import java.time.Instant
 
 
@@ -45,8 +47,7 @@ fun Routing.wireGatewayApi(db: Database, cfg: NexusConfig) = authApi(cfg.wireGat
         cfg.checkCurrency(req.amount)
         req.credit_account.expectRequestIban()
         val bankId = run {
-            val bytes = ByteArray(16)
-            kotlin.random.Random.nextBytes(bytes)
+            val bytes = ByteArray(16).rand()
             Base32Crockford.encode(bytes)
         }
         val res = db.exchange.transfer(
@@ -76,7 +77,7 @@ fun Routing.wireGatewayApi(db: Database, cfg: NexusConfig) = authApi(cfg.wireGat
         if (items.isEmpty()) {
             call.respond(HttpStatusCode.NoContent)
         } else {
-            call.respond(reduce(items, cfg.payto))
+            call.respond(reduce(items, cfg.ebics.payto))
         }
     }
     get("/taler-wire-gateway/history/incoming") {
@@ -91,8 +92,7 @@ fun Routing.wireGatewayApi(db: Database, cfg: NexusConfig) = authApi(cfg.wireGat
         req.debit_account.expectRequestIban()
         val timestamp = Instant.now()
         val bankId = run {
-            val bytes = ByteArray(16)
-            kotlin.random.Random.nextBytes(bytes)
+            val bytes = ByteArray(16).rand()
             Base32Crockford.encode(bytes)
         }
         val res = db.payment.registerTalerableIncoming(IncomingPayment(
