@@ -19,8 +19,8 @@
 
 package tech.libeufin.nexus.db
 
-import tech.libeufin.common.db.*
 import tech.libeufin.common.*
+import tech.libeufin.common.db.*
 import tech.libeufin.nexus.IncomingPayment
 import tech.libeufin.nexus.OutgoingPayment
 import java.time.Instant
@@ -39,30 +39,24 @@ class PaymentDAO(private val db: Database) {
         paymentData: OutgoingPayment, 
         wtid: ShortHashCode?,
         baseUrl: ExchangeUrl?,
-    ): OutgoingRegistrationResult = db.conn {        
-        val stmt = it.prepareStatement("""
-            SELECT out_tx_id, out_initiated, out_found
-            FROM register_outgoing((?,?)::taler_amount,?,?,?,?,?,?)
-        """)
+    ): OutgoingRegistrationResult = db.serializable(
+        """
+        SELECT out_tx_id, out_initiated, out_found
+        FROM register_outgoing((?,?)::taler_amount,?,?,?,?,?,?)
+        """
+    ) {
         val executionTime = paymentData.executionTime.micros()
-        stmt.setLong(1, paymentData.amount.value)
-        stmt.setInt(2, paymentData.amount.frac)
-        stmt.setString(3, paymentData.wireTransferSubject)
-        stmt.setLong(4, executionTime)
-        stmt.setString(5, paymentData.creditPaytoUri)
-        stmt.setString(6, paymentData.messageId)
-        if (wtid != null) {
-            stmt.setBytes(7, wtid.raw)
-        } else {
-            stmt.setNull(7, java.sql.Types.NULL)
-        }
-        if (baseUrl != null) {
-            stmt.setString(8, baseUrl.url)
-        } else {
-            stmt.setNull(8, java.sql.Types.NULL)
-        }
+
+        setLong(1, paymentData.amount.value)
+        setInt(2, paymentData.amount.frac)
+        setString(3, paymentData.wireTransferSubject)
+        setLong(4, executionTime)
+        setString(5, paymentData.creditPaytoUri)
+        setString(6, paymentData.messageId)
+        setBytes(7, wtid?.raw)
+        setString(8, baseUrl?.url)
   
-        stmt.one {
+        one {
             OutgoingRegistrationResult(
                 it.getLong("out_tx_id"),
                 it.getBoolean("out_initiated"),
@@ -82,24 +76,23 @@ class PaymentDAO(private val db: Database) {
     suspend fun registerMalformedIncoming(
         paymentData: IncomingPayment,
         bounceAmount: TalerAmount,
-        now: Instant
-    ): IncomingBounceRegistrationResult = db.conn {       
-        val stmt = it.prepareStatement("""
-            SELECT out_found, out_tx_id, out_bounce_id
-            FROM register_incoming_and_bounce((?,?)::taler_amount,?,?,?,?,(?,?)::taler_amount,?)
-        """)
-        val refundTimestamp = now.micros()
-        val executionTime = paymentData.executionTime.micros()
-        stmt.setLong(1, paymentData.amount.value)
-        stmt.setInt(2, paymentData.amount.frac)
-        stmt.setString(3, paymentData.wireTransferSubject)
-        stmt.setLong(4, executionTime)
-        stmt.setString(5, paymentData.debitPaytoUri)
-        stmt.setString(6, paymentData.bankId)
-        stmt.setLong(7, bounceAmount.value)
-        stmt.setInt(8, bounceAmount.frac)
-        stmt.setLong(9, refundTimestamp)
-        stmt.one {
+        timestamp: Instant
+    ): IncomingBounceRegistrationResult = db.serializable(
+        """
+        SELECT out_found, out_tx_id, out_bounce_id
+        FROM register_incoming_and_bounce((?,?)::taler_amount,?,?,?,?,(?,?)::taler_amount,?)
+        """
+    ) {
+        setLong(1, paymentData.amount.value)
+        setInt(2, paymentData.amount.frac)
+        setString(3, paymentData.wireTransferSubject)
+        setLong(4, paymentData.executionTime.micros())
+        setString(5, paymentData.debitPaytoUri)
+        setString(6, paymentData.bankId)
+        setLong(7, bounceAmount.value)
+        setInt(8, bounceAmount.frac)
+        setLong(9, timestamp.micros())
+        one {
             IncomingBounceRegistrationResult(
                 it.getLong("out_tx_id"),
                 it.getString("out_bounce_id"),
@@ -118,20 +111,21 @@ class PaymentDAO(private val db: Database) {
     suspend fun registerTalerableIncoming(
         paymentData: IncomingPayment,
         reservePub: EddsaPublicKey
-    ): IncomingRegistrationResult = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT out_reserve_pub_reuse, out_found, out_tx_id
-            FROM register_incoming_and_talerable((?,?)::taler_amount,?,?,?,?,?)
-        """)
+    ): IncomingRegistrationResult = db.serializable(
+        """
+        SELECT out_reserve_pub_reuse, out_found, out_tx_id
+        FROM register_incoming_and_talerable((?,?)::taler_amount,?,?,?,?,?)
+        """
+    ) {
         val executionTime = paymentData.executionTime.micros()
-        stmt.setLong(1, paymentData.amount.value)
-        stmt.setInt(2, paymentData.amount.frac)
-        stmt.setString(3, paymentData.wireTransferSubject)
-        stmt.setLong(4, executionTime)
-        stmt.setString(5, paymentData.debitPaytoUri)
-        stmt.setString(6, paymentData.bankId)
-        stmt.setBytes(7, reservePub.raw)
-        stmt.one {
+        setLong(1, paymentData.amount.value)
+        setInt(2, paymentData.amount.frac)
+        setString(3, paymentData.wireTransferSubject)
+        setLong(4, executionTime)
+        setString(5, paymentData.debitPaytoUri)
+        setString(6, paymentData.bankId)
+        setBytes(7, reservePub.raw)
+        one {
             when {
                 it.getBoolean("out_reserve_pub_reuse") -> IncomingRegistrationResult.ReservePubReuse
                 else -> IncomingRegistrationResult.Success(
@@ -145,19 +139,20 @@ class PaymentDAO(private val db: Database) {
     /** Register an incoming payment */
     suspend fun registerIncoming(
         paymentData: IncomingPayment
-    ): IncomingRegistrationResult.Success = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT out_found, out_tx_id
-            FROM register_incoming((?,?)::taler_amount,?,?,?,?)
-        """)
+    ): IncomingRegistrationResult.Success = db.serializable(
+        """
+        SELECT out_found, out_tx_id
+        FROM register_incoming((?,?)::taler_amount,?,?,?,?)
+        """
+    ) { 
         val executionTime = paymentData.executionTime.micros()
-        stmt.setLong(1, paymentData.amount.value)
-        stmt.setInt(2, paymentData.amount.frac)
-        stmt.setString(3, paymentData.wireTransferSubject)
-        stmt.setLong(4, executionTime)
-        stmt.setString(5, paymentData.debitPaytoUri)
-        stmt.setString(6, paymentData.bankId)
-        stmt.one {
+        setLong(1, paymentData.amount.value)
+        setInt(2, paymentData.amount.frac)
+        setString(3, paymentData.wireTransferSubject)
+        setLong(4, executionTime)
+        setString(5, paymentData.debitPaytoUri)
+        setString(6, paymentData.bankId)
+        one {
             IncomingRegistrationResult.Success(
                 it.getLong("out_tx_id"),
                 !it.getBoolean("out_found")
@@ -189,21 +184,22 @@ class PaymentDAO(private val db: Database) {
         }
 
     /** List incoming transaction metadata for debugging */
-    suspend fun metadataIncoming(): List<IncomingTxMetadata> = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-                (amount).val as amount_val
-                ,(amount).frac AS amount_frac
-                ,wire_transfer_subject
-                ,execution_time
-                ,debit_payto_uri
-                ,bank_id
-                ,reserve_public_key
-            FROM incoming_transactions
-                LEFT OUTER JOIN talerable_incoming_transactions using (incoming_transaction_id)
-            ORDER BY execution_time
-        """)
-        stmt.all {
+    suspend fun metadataIncoming(): List<IncomingTxMetadata> = db.serializable(
+        """
+        SELECT
+            (amount).val as amount_val
+            ,(amount).frac AS amount_frac
+            ,wire_transfer_subject
+            ,execution_time
+            ,debit_payto_uri
+            ,bank_id
+            ,reserve_public_key
+        FROM incoming_transactions
+            LEFT OUTER JOIN talerable_incoming_transactions using (incoming_transaction_id)
+        ORDER BY execution_time
+        """
+    ) {
+        all {
             IncomingTxMetadata(
                 date = it.getLong("execution_time").asInstant(),
                 amount = it.getDecimal("amount"),
@@ -216,22 +212,23 @@ class PaymentDAO(private val db: Database) {
     }
 
     /** List outgoing transaction metadata for debugging */
-    suspend fun metadataOutgoing(): List<OutgoingTxMetadata> = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-                (amount).val as amount_val
-                ,(amount).frac AS amount_frac
-                ,wire_transfer_subject
-                ,execution_time
-                ,credit_payto_uri
-                ,message_id
-                ,wtid
-                ,exchange_base_url
-            FROM outgoing_transactions
-                LEFT OUTER JOIN talerable_outgoing_transactions using (outgoing_transaction_id)
-            ORDER BY execution_time
-        """)
-        stmt.all {
+    suspend fun metadataOutgoing(): List<OutgoingTxMetadata> = db.serializable(
+        """
+        SELECT
+            (amount).val as amount_val
+            ,(amount).frac AS amount_frac
+            ,wire_transfer_subject
+            ,execution_time
+            ,credit_payto_uri
+            ,message_id
+            ,wtid
+            ,exchange_base_url
+        FROM outgoing_transactions
+            LEFT OUTER JOIN talerable_outgoing_transactions using (outgoing_transaction_id)
+        ORDER BY execution_time
+        """
+    ) {
+        all {
             OutgoingTxMetadata(
                 date = it.getLong("execution_time").asInstant(),
                 amount = it.getDecimal("amount"),
@@ -245,23 +242,24 @@ class PaymentDAO(private val db: Database) {
     }
 
     /** List initiated transaction metadata for debugging */
-    suspend fun metadataInitiated(): List<InitiatedTxMetadata> = db.conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-                (amount).val as amount_val
-                ,(amount).frac AS amount_frac
-                ,wire_transfer_subject
-                ,initiation_time
-                ,last_submission_time
-                ,submission_counter
-                ,credit_payto_uri
-                ,submitted
-                ,request_uid
-                ,failure_message
-            FROM initiated_outgoing_transactions
-            ORDER BY initiation_time
-        """)
-        stmt.all {
+    suspend fun metadataInitiated(): List<InitiatedTxMetadata> = db.serializable(
+        """
+        SELECT
+            (amount).val as amount_val
+            ,(amount).frac AS amount_frac
+            ,wire_transfer_subject
+            ,initiation_time
+            ,last_submission_time
+            ,submission_counter
+            ,credit_payto_uri
+            ,submitted
+            ,request_uid
+            ,failure_message
+        FROM initiated_outgoing_transactions
+        ORDER BY initiation_time
+        """
+    ) {
+        all {
             InitiatedTxMetadata(
                 date = it.getLong("initiation_time").asInstant(),
                 amount = it.getDecimal("amount"),
@@ -283,7 +281,7 @@ data class IncomingTxMetadata(
     val amount: DecimalNumber,
     val subject: String,
     val debtor: String,
-    val id: String,
+    val id: String?,
     val reservePub: EddsaPublicKey?
 )
 
