@@ -20,7 +20,7 @@
 package tech.libeufin.nexus.ebics
 
 import org.w3c.dom.Document
-import tech.libeufin.nexus.NexusConfig
+import tech.libeufin.nexus.NexusEbicsConfig
 import tech.libeufin.nexus.XmlBuilder
 import tech.libeufin.nexus.XmlDestructor
 import java.io.InputStream
@@ -45,7 +45,7 @@ data class OrderInfo (
 )
 
 object EbicsAdministrative {
-    fun HEV(cfg: NexusConfig): ByteArray {
+    fun HEV(cfg: NexusEbicsConfig): ByteArray {
         return XmlBuilder.toBytes("ebicsHEVRequest") {
             attr("xmlns", "http://www.ebics.org/H000")
             el("HostID", cfg.ebicsHostId)
@@ -86,7 +86,7 @@ object EbicsAdministrative {
                     OrderInfo(
                         one("AdminOrderType").text(),
                         opt("Service") {
-                            var params = StringBuilder()
+                            val params = StringBuilder()
                             opt("ServiceName")?.run {
                                 params.append(" ${text()}")
                             }
