@@ -1,6 +1,6 @@
 /*
  * This file is part of LibEuFin.
- * Copyright (C) 2023 Stanisci and Dold.
+ * Copyright (C) 2023-2024 Taler Systems S.A.
 
  * LibEuFin is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -21,13 +21,14 @@ import com.github.ajalt.clikt.core.CliktCommand
 import com.github.ajalt.clikt.testing.test
 import tech.libeufin.common.crypto.CryptoUtil
 import tech.libeufin.nexus.*
+import tech.libeufin.nexus.cli.LibeufinNexus
 import java.io.ByteArrayOutputStream
 import java.io.PrintStream
 import kotlin.io.path.*
 import kotlin.test.Test
 import kotlin.test.assertEquals
 
-val nexusCmd = LibeufinNexusCommand()
+val nexusCmd = LibeufinNexus()
 
 fun CliktCommand.testErr(cmd: String, msg: String) {
     val prevOut = System.err
@@ -50,9 +51,10 @@ class CliTest {
         val cmds = listOf("ebics-submit", "ebics-fetch")
         val allCmds = listOf("ebics-submit", "ebics-fetch", "ebics-setup")
         val conf = "conf/test.conf"
-        val cfg = loadConfig(Path(conf))
-        val clientKeysPath = cfg.requirePath("nexus-ebics", "client_private_keys_file")
-        val bankKeysPath = cfg.requirePath("nexus-ebics", "bank_public_keys_file")
+        val nexusCfg = nexusConfig(Path(conf))
+        val cfg = nexusCfg.ebics
+        val clientKeysPath = cfg.clientPrivateKeysPath
+        val bankKeysPath = cfg.bankPublicKeysPath
         clientKeysPath.parent!!.createDirectories()
         clientKeysPath.parent!!.toFile().setWritable(true)
         bankKeysPath.parent!!.createDirectories()
@@ -118,4 +120,41 @@ class CliTest {
             nexusCmd.testErr("ebics-setup -c $conf", "Could not write client private keys at '$clientKeysPath': permission denied on '${clientKeysPath.parent}'")
         }
     }
+
+    /** Test server check */
+    @Test
+    fun serveCheck() {
+        val confs = listOf(
+            "mini" to 1,
+            "test" to 0
+        )
+        for ((conf, statusCode) in confs) {
+            val result = nexusCmd.test("serve --check -c conf/$conf.conf")
+            assertEquals(statusCode, result.statusCode)
+        }
+    }
+
+    /** Test list cmds */
+    @Test
+    fun listCheck() = setup { db, _ ->
+        fun check() {
+            for (list in listOf("incoming", "outgoing", "initiated")) {
+                val result = nexusCmd.test("testing list $list -c conf/test.conf")
+                assertEquals(0, result.statusCode)
+            }
+        }
+        // Check empty
+        check()
+        // Check with transactions
+        ingestIn(db)
+        ingestOut(db)
+        check()
+        // Check with taler transactions
+        talerableOut(db)
+        talerableIn(db)
+        check()
+        // Check with null id
+        talerableIn(db, true)
+        check()
+    }
 }
\ No newline at end of file
