@@ -18,15 +18,16 @@
  */
 
 import org.junit.Test
-import tech.libeufin.common.*
+import tech.libeufin.common.ShortHashCode
+import tech.libeufin.common.TalerAmount
+import tech.libeufin.common.db.one
+import tech.libeufin.nexus.AccountType
+import tech.libeufin.nexus.cli.ingestIncomingPayment
+import tech.libeufin.nexus.cli.ingestOutgoingPayment
+import tech.libeufin.nexus.db.Database
 import tech.libeufin.nexus.db.InitiatedDAO.PaymentInitiationResult
-import tech.libeufin.nexus.*
 import java.time.Instant
-import kotlin.test.assertEquals
-import kotlin.test.assertIs
-import kotlin.test.assertFalse
-import kotlin.test.assertNull
-import kotlin.test.assertTrue
+import kotlin.test.*
 
 class OutgoingPaymentsTest {
     @Test
@@ -71,8 +72,25 @@ class OutgoingPaymentsTest {
     }
 }
 
+suspend fun Database.checkCount(nbIncoming: Int, nbBounce: Int, nbTalerable: Int) {
+    conn {
+        val cIncoming = it.prepareStatement("SELECT count(*) FROM incoming_transactions").one { it.getInt(1) }
+        val cBounce = it.prepareStatement("SELECT count(*) FROM bounced_transactions").one { it.getInt(1) }
+        val cTalerable = it.prepareStatement("SELECT count(*) FROM talerable_incoming_transactions").one { it.getInt(1) }
+        assertEquals(Triple(nbIncoming, nbBounce, nbTalerable), Triple(cIncoming, cBounce, cTalerable))
+    }
+}
+
+suspend fun Database.inTxExists(id: String): Boolean = conn {
+    it.prepareStatement("SELECT EXISTS(SELECT FROM incoming_transactions WHERE bank_id = ?)").apply {
+        setString(1, id)
+    }.one {
+        it.getBoolean(1)
+    }
+}
+
 class IncomingPaymentsTest {
-    // Tests creating and bouncing incoming payments in one DB transaction.
+    // Tests creating and bouncing incoming payments in one DB transaction
     @Test
     fun bounce() = setup { db, _ -> 
         // creating and bouncing one incoming transaction.
@@ -125,6 +143,46 @@ class IncomingPaymentsTest {
             )
         }
     }
+
+    // Test creating an incoming taler transaction without and ID and reconcile it later again
+    @Test
+    fun reconcileMissingId() = setup { db, _ ->
+        // Register with missing ID
+        val reserve_pub = ShortHashCode.rand()
+        val incoming = genInPay("history test with $reserve_pub reserve pub")
+        val incomingMissingId = incoming.copy(bankId = null)
+        ingestIncomingPayment(db, incomingMissingId, AccountType.exchange)
+        db.checkCount(1, 0, 1)
+        assertFalse(db.inTxExists(incoming.bankId!!))
+
+        // Idempotent
+        ingestIncomingPayment(db, incomingMissingId, AccountType.exchange)
+        db.checkCount(1, 0, 1)
+
+        // Different metadata is bounced
+        ingestIncomingPayment(db, genInPay("another $reserve_pub reserve pub"), AccountType.exchange)
+        db.checkCount(2, 1, 1)
+
+        // Different medata with missing id is ignored
+        ingestIncomingPayment(db, incomingMissingId.copy(amount = TalerAmount("KUDOS:9")), AccountType.exchange)
+        db.checkCount(2, 1, 1)
+
+        // Recover bank ID when metadata match
+        ingestIncomingPayment(db, incoming, AccountType.exchange)
+        assertTrue(db.inTxExists(incoming.bankId!!))
+
+        // Idempotent
+        ingestIncomingPayment(db, incoming, AccountType.exchange)
+        db.checkCount(2, 1, 1)
+        
+        // Missing ID is ignored
+        ingestIncomingPayment(db, incomingMissingId, AccountType.exchange)
+        db.checkCount(2, 1, 1)
+
+        // Other ID is bounced known that we know the id
+        ingestIncomingPayment(db, incomingMissingId.copy(bankId = "NEW"), AccountType.exchange)
+        db.checkCount(3, 2, 1)
+    }
 }
 class PaymentInitiationsTest {
 
@@ -224,4 +282,23 @@ class PaymentInitiationsTest {
             db.initiated.submittable("KUDOS").map { it.requestUid }
         )
     }
+}
+
+class EbicsTxTest {
+    // Test pending transaction's id
+    @Test
+    fun pending() = setup { db, _ ->
+        val ids = setOf("first", "second", "third")
+        for (id in ids) {
+            db.ebics.register(id)
+        }
+
+        repeat(ids.size) {
+            val id = db.ebics.first()
+            assert(ids.contains(id))
+            db.ebics.remove(id!!)
+        }
+
+        assertNull(db.ebics.first())
+    }
 }
\ No newline at end of file
