@@ -27,10 +27,20 @@ import io.ktor.client.statement.*
 import io.ktor.http.*
 import kotlinx.coroutines.runBlocking
 import org.junit.Test
-import tech.libeufin.bank.*
+import tech.libeufin.bank.BankAccountTransactionsResponse
+import tech.libeufin.bank.CashoutResponse
+import tech.libeufin.bank.ConversionResponse
+import tech.libeufin.bank.RegisterAccountResponse
+import tech.libeufin.bank.cli.LibeufinBank
 import tech.libeufin.common.*
+import tech.libeufin.common.api.engine
 import tech.libeufin.common.db.one
-import tech.libeufin.nexus.*
+import tech.libeufin.nexus.AccountType
+import tech.libeufin.nexus.IncomingPayment
+import tech.libeufin.nexus.cli.LibeufinNexus
+import tech.libeufin.nexus.cli.ingestIncomingPayment
+import tech.libeufin.nexus.nexusConfig
+import tech.libeufin.nexus.withDb
 import java.time.Instant
 import kotlin.io.path.Path
 import kotlin.io.path.readText
@@ -68,9 +78,8 @@ fun server(lambda: () -> Unit) {
 fun setup(conf: String, lambda: suspend (NexusDb) -> Unit) {
     try {
         runBlocking {
-            val cfg = loadConfig(Path(conf)).dbConfig()
-            NexusDb(cfg).use {
-                lambda(it)
+            nexusConfig(Path(conf)).withDb { db, _ ->
+                lambda(db)
             }
         }
     } finally {
@@ -88,8 +97,8 @@ inline fun assertException(msg: String, lambda: () -> Unit) {
 }
 
 class IntegrationTest {
-    val nexusCmd = LibeufinNexusCommand()
-    val bankCmd = LibeufinBankCommand()
+    val nexusCmd = LibeufinNexus()
+    val bankCmd = LibeufinBank()
     val client = HttpClient(CIO)
 
     @Test
@@ -103,12 +112,17 @@ class IntegrationTest {
             bankCmd.run("serve $flags")
         }
         
-        setup("conf/mini.conf") { _ ->
+        setup("conf/mini.conf") {
             // Check bank is running
             client.get("http://0.0.0.0:8080/public-accounts").assertNoContent()
         }
 
         bankCmd.run("gc $flags")
+
+        server {
+            nexusCmd.run("serve $flags")
+        }
+        engine?.stop(0, 0) 
     }
 
     @Test
@@ -127,7 +141,7 @@ class IntegrationTest {
             }
         }
 
-        setup("conf/integration.conf")  { db ->
+        setup("conf/integration.conf") { db ->
             val userPayTo = IbanPayto.rand()
             val fiatPayTo = IbanPayto.rand()
     
@@ -148,14 +162,14 @@ class IntegrationTest {
             )
 
             assertException("ERROR: cashin failed: missing exchange account") {
-                ingestIncomingPayment(db, payment)
+                ingestIncomingPayment(db, payment, AccountType.exchange)
             }
 
             // Create exchange account
             bankCmd.run("create-account $flags -u exchange -p password --name 'Mr Money' --exchange")
     
             assertException("ERROR: cashin currency conversion failed: missing conversion rates") {
-                ingestIncomingPayment(db, payment)
+                ingestIncomingPayment(db, payment, AccountType.exchange)
             }
 
             // Start server
@@ -191,7 +205,7 @@ class IntegrationTest {
             checkCount(db, 0, 0, 0)
             ingestIncomingPayment(db, payment.copy(
                 amount = TalerAmount("EUR:0.01"),
-            ))
+            ), AccountType.exchange)
             checkCount(db, 1, 1, 0)
             client.get("http://0.0.0.0:8080/accounts/exchange/transactions") {
                 basicAuth("exchange", "password")
@@ -205,14 +219,14 @@ class IntegrationTest {
                 executionTime = Instant.now(),
                 bankId = "success"
             )
-            ingestIncomingPayment(db, valid_payment)
+            ingestIncomingPayment(db, valid_payment, AccountType.exchange)
             checkCount(db, 2, 1, 1)
             client.get("http://0.0.0.0:8080/accounts/exchange/transactions") {
                 basicAuth("exchange", "password")
             }.assertOkJson<BankAccountTransactionsResponse>()
 
             // Check idempotency
-            ingestIncomingPayment(db, valid_payment)
+            ingestIncomingPayment(db, valid_payment, AccountType.exchange)
             checkCount(db, 2, 1, 1)
             // TODO check double insert cashin with different subject
         }
